@@ -396,13 +396,34 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
   double nclip_low = 3.0, nclip_up = 3.0;
   /*-------------------------------------------------------------------------*/
 
+  float *lbda = cpl_table_get_data_float(aPixtable->table, MUSE_PIXTABLE_LAMBDA);
   cpl_boolean isAO = muse_pfits_get_mode(aPixtable->header)
                      >= MUSE_MODE_WFM_AO_E;
   if (isAO) {
     /* only one range around the NaD gap, remove the 5850 entry */
     memmove(lbdabins+5, lbdabins+6, 15 * sizeof(float));
     nlbin--;
-  }
+
+    if (getenv("MUSE_AUTOCAL_EXCLUDE_RAMAN") &&
+        atof(getenv("MUSE_AUTOCAL_EXCLUDE_RAMAN")) > 0.) {
+      double wrange = atof(getenv("MUSE_AUTOCAL_EXCLUDE_RAMAN"));
+      /* also take the ranges around the Raman peaks out of *
+       * the computation, by unselecting those table rows   */
+      cpl_size irow, nrow = muse_pixtable_get_nrow(aPixtable),
+               nsel = cpl_table_count_selected(aPixtable->table);
+      const float lO2 = 6483.8, lN2 = 6826.5;
+      for (irow = 0; irow < nrow; irow++) {
+        /* approximate central peaks of the Raman features */
+        if ((lbda[irow] > lO2 - wrange && lbda[irow] < lO2 + wrange) |
+            (lbda[irow] > lN2 - wrange && lbda[irow] < lN2 + wrange)) {
+          cpl_table_unselect_row(aPixtable->table, irow);
+        }
+      } /* for irow (all table rows) */
+      cpl_size nsel2 = cpl_table_count_selected(aPixtable->table);
+      cpl_msg_debug(__func__, "Excluded %"CPL_SIZE_FORMAT" pixels %.3f Angstrom"
+                    " around the Raman peaks", nsel - nsel2, wrange);
+    } /* if envvar set */
+  } /* if isAO */
 
   cpl_msg_info(__func__, "Running self-calibration, using %d lambda ranges "
                "(%s mode)", nlbin, isAO ? "AO" : "non-AO");
@@ -437,8 +458,7 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
   }
 
   float *data = cpl_table_get_data_float(aPixtable->table, MUSE_PIXTABLE_DATA),
-        *stat = cpl_table_get_data_float(aPixtable->table, MUSE_PIXTABLE_STAT),
-        *lbda = cpl_table_get_data_float(aPixtable->table, MUSE_PIXTABLE_LAMBDA);
+        *stat = cpl_table_get_data_float(aPixtable->table, MUSE_PIXTABLE_STAT);
 
   cpl_boolean debug = getenv("MUSE_DEBUG_AUTOCALIB") ? CPL_TRUE : CPL_FALSE;
 
@@ -496,9 +516,9 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
         k = muse_mapidx(i+1, s+1, q+1);
         int slidx = kMuseSlicesPerCCD * i + s;
         if (npts[k] > 100) {
-          /* Compute the mean flux of the ifu/slice/wavelength bin,  *
-           * if we have at least 100 values, and if we have at least *
-           * min_pix_per_slice spaxels with a flux.                  */
+          /* Compute the mean flux of the ifu/slice/wavelength bin, *
+           * if we have at least 50 values, and if we have at least *
+           * min_pix_per_slice spaxels with a flux.                 */
           muse_autocalib_slice_mean(data, xpix, npts[k], x, indmap[k],
                                     min_pix_per_slice);
           if (x[2] > min_pix_per_slice) {
@@ -522,7 +542,7 @@ muse_autocalib_slice_median(muse_pixtable *aPixtable, double aClip,
       } /* for s */
 
       /* Compute the mean flux for each half-IFU, as the mean of the *
-       * slice fluxes. */
+       * slice fluxes.                                               */
       if (!slice_count1) {
         ifu_flux[2 * i] = 0.0;
       } else {
