@@ -33,6 +33,7 @@
 #include "muse_datacube.h"
 
 #include "muse_flux.h"
+#include "muse_idp.h"
 #include "muse_pfits.h"
 #include "muse_quality.h"
 #include "muse_utils.h"
@@ -226,7 +227,9 @@ muse_euro3dcube_collapse(muse_euro3dcube *aEuro3D, const muse_table *aFilter)
   /* create output muse_image, but only with two image extensions */
   muse_image *image = muse_image_new();
   image->header = cpl_propertylist_duplicate(aEuro3D->header);
-  /* the axis3 WCS was already erased in muse_resampling_euro3d() */
+  /* the axis3 WCS was already erased in muse_resampling_euro3d(), thus    *
+   * just get rid of the remaining keywords referring to the spectral axis */
+  cpl_propertylist_erase_regexp(image->header, "^SPECSYS$", 0);
   if (aFilter) {
     muse_utils_filter_copy_properties(image->header, aFilter, ffraction);
   }
@@ -247,7 +250,14 @@ muse_euro3dcube_collapse(muse_euro3dcube *aEuro3D, const muse_table *aFilter)
 
   /* loop through all spaxels and over all wavelengths */
   int k, noutside = 0;
-  #pragma omp parallel for default(none) private(l)      /* as req. by Ralf */ \
+
+  // FIXME: Removed default(none) clause here, to make the code work with
+  //        gcc9 while keeping older versions of gcc working too without
+  //        resorting to conditional compilation. Having default(none) here
+  //        causes gcc9 to abort the build because of __func__ not being
+  //        specified in a data sharing clause. Adding a data sharing clause
+  //        makes older gcc versions choke.
+  #pragma omp parallel for private(l)                                       \
           shared(aEuro3D, fdata, l1, l2, noutside, nspec, nx, ny, outdata,     \
                  outdq, usevariance, wcs)
   for (k = 0; k < nspec; k++) {
@@ -392,7 +402,7 @@ muse_datacube_collapse(muse_datacube *aCube, const muse_table *aFilter)
   /* create output muse_image, but only with two image extensions */
   muse_image *image = muse_image_new();
   image->header = cpl_propertylist_duplicate(aCube->header);
-  cpl_propertylist_erase_regexp(image->header, "^C...*3$|^CD3_.$", 0);
+  cpl_propertylist_erase_regexp(image->header, "^C...*3$|^CD3_.$|^SPECSYS$", 0);
   if (aFilter) {
     muse_utils_filter_copy_properties(image->header, aFilter, ffraction);
   }
@@ -541,7 +551,7 @@ muse_datacube_save_recimages(const char *aFilename, muse_imagelist *aImages,
       cpl_propertylist_set_comment(header, "EXTNAME", "reconstructed image (variance)");
       if (unit) {
         char *ustat = cpl_sprintf("(%s)**2", unit); /* variance in squared units */
-        cpl_propertylist_append_string(header, "BUNIT", ustat);
+        cpl_propertylist_update_string(header, "BUNIT", ustat);
         cpl_free(ustat);
       }
       snprintf(obj, KEYWORD_LENGTH, "%s, %s", cpl_array_get_string(aNames, i),
@@ -755,7 +765,7 @@ muse_datacube_save(muse_datacube *aCube, const char *aFilename)
   cpl_propertylist *header = cpl_propertylist_new();
   /* copy all headers, except the main WCS keys */
   cpl_propertylist_copy_property_regexp(header, aCube->header,
-                                        MUSE_WCS_KEYS, 1);
+                                        MUSE_WCS_KEYS"|^BUNIT", 1);
   cpl_error_code rc = cpl_propertylist_save(header, aFilename, CPL_IO_CREATE);
   cpl_propertylist_delete(header);
 
@@ -793,10 +803,22 @@ muse_datacube_save(muse_datacube *aCube, const char *aFilename)
     cpl_propertylist_append_string(header, "EXTNAME", EXTNAME_STAT);
     cpl_propertylist_set_comment(header, "EXTNAME", EXTNAME_STAT_COMMENT);
     /* add the correct BUNIT, depending on the data units */
-    if (!strncmp(muse_pfits_get_bunit(aCube->header),
-                 kMuseFluxUnitString, strlen(kMuseFluxUnitString) + 1)) {
+    const char *bunit = muse_pfits_get_bunit(aCube->header);
+    if (bunit) {
       /* flux calibrated data */
-      cpl_propertylist_append_string(header, "BUNIT", kMuseFluxStatString);
+      const char *bunitStat = NULL;
+      if (!strncmp(bunit, kMuseFluxUnitString,
+                   strlen(kMuseFluxUnitString) + 1)) {
+        /* standard unit syntax */
+        bunitStat = kMuseFluxStatString;
+      } else if (!strncmp(bunit, kMuseIdpFluxDataUnit,
+                          strlen(kMuseIdpFluxDataUnit) + 1)) {
+        /* IDP unit syntax */
+        bunitStat = kMuseIdpFluxStatUnit;
+      }
+      if (bunitStat) {
+        cpl_propertylist_append_string(header, "BUNIT", bunitStat);
+      }
     }
     muse_utils_copy_modified_header(aCube->header, header, "OBJECT",
                                     EXTNAME_STAT);
@@ -833,7 +855,7 @@ muse_datacube_load_header(const char *aFilename)
   cpl_propertylist *header = cpl_propertylist_load(aFilename, 0);
   cpl_propertylist *hdata = cpl_propertylist_load(aFilename, extdata);
   cpl_propertylist_copy_property_regexp(header, hdata,
-                                        MUSE_WCS_KEYS"|BUNIT", 0);
+                                        MUSE_WCS_KEYS"|^BUNIT", 0);
   cpl_propertylist_delete(hdata);
   return header;
 } /* muse_datacube_load_header() */
