@@ -899,7 +899,7 @@ muse_find_stars(const cpl_image *aImage, double aHmin, double aFwhm,
                         _cpl_matrix_sum(mtmp) / npixels) / flux;
     cpl_matrix_delete(mtmp);
 
-    if ((sharpness < kSharpLimits[0]) || (sharpness > kSharpLimits[1])) {
+    if ((sharpness < aSharpLimits[0]) || (sharpness > aSharpLimits[1])) {
       cpl_matrix_unwrap(mimg);
       cpl_image_delete(_image);
 
@@ -931,7 +931,7 @@ muse_find_stars(const cpl_image *aImage, double aHmin, double aFwhm,
     }
 
     double roundness = 2. * (dx - dy) / (dx + dy);
-    if ((roundness < kRoundLimits[0]) || (roundness > kRoundLimits[1])) {
+    if ((roundness < aRoundLimits[0]) || (roundness > aRoundLimits[1])) {
       cpl_matrix_unwrap(mimg);
       cpl_image_delete(_image);
 
