@@ -124,9 +124,11 @@ typedef enum {
 /*----------------------------------------------------------------------------*/
 typedef enum {
   MUSE_FLUX_PROFILE_MOFFAT = 0, /**< Moffat profile */
+  MUSE_FLUX_PROFILE_SMOFFAT, /**< "smoothed" Moffat profile */
   MUSE_FLUX_PROFILE_GAUSSIAN, /**< Gaussian profile */
   MUSE_FLUX_PROFILE_CIRCLE, /**< circular flux integration, with background annulus */
-  MUSE_FLUX_PROFILE_EQUAL_SQUARE /**< simplest version: non-optimal square integration */
+  MUSE_FLUX_PROFILE_EQUAL_SQUARE, /**< simplest version: non-optimal square integration */
+  MUSE_FLUX_PROFILE_AUTO /**< automatic best selection (smoffat for WFM, circle for NFM) */
 } muse_flux_profile_type;
 
 /*----------------------------------------------------------------------------*/
