@@ -89,9 +89,9 @@
 #define KEY_SPEC_RES                 "SPEC_RES"
 #define KEY_SPEC_RES_COMMENT         "Spectral resolving power at central wavelength"
 #define KEY_SPEC_ERR                 "CRDER3"
-#define KEY_SPEC_ERR_COMMENT         "[Angstrom] Random error in spectral coordinate"
+#define KEY_SPEC_ERR_COMMENT         "[angstrom] Random error in spectral coordinate"
 #define KEY_PIXNOISE                 "PIXNOISE"
-#define KEY_PIXNOISE_COMMENT         "[erg/s/cm**2/Angstrom] pixel-to-pixel noise"
+#define KEY_PIXNOISE_COMMENT         "[erg.s**(-1).cm**(-2).angstrom**(-1)] pixel-to-pixel noise"
 #define KEY_ABMAGLIM                 "ABMAGLIM"
 #define KEY_ABMAGLIM_COMMENT         "5-sigma magnitude limit for point sources"
 #define KEY_NCOMBINE                 "NCOMBINE"
@@ -980,12 +980,17 @@ muse_idp_properties_collect(muse_processing *aProcessing,
 
   /* Estimate exposure time per pixel. If the number of illuminated pixels *
    * cannot be computed, or if there are no "valid" pixels the exposure    *
-   * time is forced to zero.                                               */
+   * time is forced to zero. For single exposures the exposure time is     *
+   * forced to texptime.                                                   */
 
-  if (nptotal == 0) {
-    properties->exptime = 0.;
+  if (properties->ncombine == 1) {
+    properties->exptime = properties->texptime;
   } else {
-    properties->exptime /= nptotal;
+    if (nptotal == 0) {
+      properties->exptime = 0.;
+    } else {
+      properties->exptime /= nptotal;
+    }
   }
 
   /* Compute product image quality estimate and its uncertainty */
@@ -1123,7 +1128,7 @@ muse_idp_properties_collect(muse_processing *aProcessing,
   }
 
   /* Get the product category from the product header, unless it is *
-   * overridden an explicitly given tag.                            */
+   * overridden by an explicitly given tag.                         */
   if (!aTag) {
     aTag = muse_pfits_get_pro_catg(header);
   }
@@ -1322,6 +1327,17 @@ muse_idp_properties_update(cpl_propertylist *aHeader,
                                        aProperties->wlerror);
   cpl_propertylist_set_comment(aHeader, KEY_SPEC_ERR, KEY_SPEC_ERR_COMMENT);
 
+  /* Update data unit values to IDP standards */
+
+  if (!strncmp(muse_pfits_get_cunit(aHeader, 3), "Angstrom", 9)) {
+    cpl_propertylist_update_string(aHeader, "CUNIT3",
+        kMuseIdpWavelengthUnit);
+  }
+  if (!strncmp(muse_pfits_get_bunit(aHeader),
+        kMuseFluxUnitString, strlen(kMuseFluxUnitString) + 1)) {
+    cpl_propertylist_update_string(aHeader, "BUNIT", kMuseIdpFluxDataUnit);
+  }
+
   /* Fallback required for IDPs */
 
   if (!cpl_propertylist_has(aHeader, "CSYER1")) {
@@ -1346,7 +1362,12 @@ muse_idp_properties_update_fov(muse_image *aFov)
   cpl_ensure(aFov, CPL_ERROR_NULL_INPUT, CPL_ERROR_NULL_INPUT);
   cpl_ensure(aFov->header, CPL_ERROR_ILLEGAL_INPUT, CPL_ERROR_ILLEGAL_INPUT);
 
-  status = cpl_propertylist_update_string(aFov->header, KEY_PRODCATG,
+  if (!strncmp(muse_pfits_get_bunit(aFov->header),
+        kMuseFluxUnitString, strlen(kMuseFluxUnitString) + 1)) {
+    status = cpl_propertylist_update_string(aFov->header, "BUNIT",
+                                            kMuseIdpFluxDataUnit);
+  }
+  status |= cpl_propertylist_update_string(aFov->header, KEY_PRODCATG,
                                            KEY_PRODCATG_VALUE_FOV_IMAGE);
   cpl_propertylist_set_comment(aFov->header, KEY_PRODCATG, KEY_PRODCATG_COMMENT);
 
