@@ -29,6 +29,16 @@
 
 #include <muse_processing.h>
 
+/*----------------------------------------------------------------------------*
+ *                              Defines                                       *
+ *----------------------------------------------------------------------------*/
+
+/* IDP unit strings */
+static const char kMuseIdpFluxDataUnit[] = "10**(-20)erg.s**(-1).cm**(-2).angstrom**(-1)";
+static const char kMuseIdpFluxStatUnit[] = "10**(-40)erg**2.s**(-2).cm**(-4).angstrom**(-2)";
+static const char kMuseIdpWavelengthUnit[] = "angstrom";
+
+
 /*----------------------------------------------------------------------------*
  *                           Data structures                                  *
  *----------------------------------------------------------------------------*/
