@@ -414,6 +414,8 @@ muse_image_save(muse_image *aImage, const char *aFilename)
   cpl_propertylist_erase(header, "BUNIT");
   /* do not save the WCS keywords, either */
   cpl_propertylist_erase_regexp(header, MUSE_WCS_KEYS, 0);
+  /* and remove ESO format headers, if they exist */
+  cpl_propertylist_erase_regexp(header, ESO_HDU_HEADERS_REGEXP, 0);
 
   /* Save the FITS header first, as the primary header of the file. *
    * The extensions will contain only minimal image headers.        */
@@ -428,6 +430,7 @@ muse_image_save(muse_image *aImage, const char *aFilename)
   /* minimal header information for the extensions to add *
    * extension name information                           */
   cpl_propertylist *extheader = cpl_propertylist_new();
+  cpl_propertylist_append_bool(extheader, "INHERIT", 1);
 
   /* copy WCS information to minimal extension header */
   cpl_propertylist_copy_property_regexp(extheader, aImage->header,
@@ -471,7 +474,7 @@ muse_image_save(muse_image *aImage, const char *aFilename)
     cpl_propertylist_set_string(extheader, "EXTNAME", EXTNAME_STAT);
     cpl_propertylist_set_comment(extheader, "EXTNAME", EXTNAME_STAT_COMMENT);
     char *ustat = cpl_sprintf("(%s)**2", unit); /* variance in squared units */
-    cpl_propertylist_append_string(extheader, "BUNIT", ustat);
+    cpl_propertylist_update_string(extheader, "BUNIT", ustat);
     cpl_free(ustat);
     muse_utils_set_hduclass(extheader, "ERROR", EXTNAME_DATA,
                             aImage->dq ? EXTNAME_DQ : NULL, EXTNAME_STAT);
@@ -936,4 +939,82 @@ muse_image_dq_to_nan(muse_image *aImage)
   return CPL_ERROR_NONE;
 } /* muse_image_dq_to_nan() */
 
+/*---------------------------------------------------------------------------*/
+/**
+  @brief    Create a mask at the border of a given image.
+  @param    aImage   the image to use for sizing the mask
+  @param    aWidth   the width in pixels of the border to mask
+  @return   a new cpl_mask on success, NULL on failure
+
+  A new cpl_mask is created, with CPL_BINARY_0 in the center, but a border
+  of size aWidth filled with CPL_BINERY_1.
+
+  @error{set CPL_ERROR_NULL_INPUT\, return NULL,
+         aImage or its data extensions are NULL}
+ */
+/*---------------------------------------------------------------------------*/
+cpl_mask *
+muse_image_create_border_mask(const muse_image *aImage, unsigned int aWidth)
+{
+  cpl_ensure(aImage && aImage->data, CPL_ERROR_NULL_INPUT, NULL);
+
+  int nx = cpl_image_get_size_x(aImage->data),
+      ny = cpl_image_get_size_y(aImage->data);
+  cpl_mask *mask = cpl_mask_new(nx, ny);
+  /* left */
+  muse_cplmask_fill_window(mask, 1, 1, aWidth, ny, CPL_BINARY_1);
+  /* right */
+  muse_cplmask_fill_window(mask, nx - aWidth + 1, 1, nx, ny, CPL_BINARY_1);
+  /* bottom */
+  muse_cplmask_fill_window(mask, 1, 1, nx, aWidth, CPL_BINARY_1);
+  /* top */
+  muse_cplmask_fill_window(mask, 1, ny - aWidth + 1, nx, ny, CPL_BINARY_1);
+
+  return mask;
+} /* muse_image_create_border_mask() */
+
+/*---------------------------------------------------------------------------*/
+/**
+  @brief    Create a mask at the border of a given image.
+  @param    aImage      the image to use for sizing the mask
+  @param    aQuadrant   the image quadrant to use for the corner mask
+  @param    aRadius     the radius in pixels from the actual image corner
+  @return   a new cpl_mask on success, NULL on failure
+
+  A new cpl_mask is created, with CPL_BINARY_0 in the center, but a corner
+  of radius aRadius filled with CPL_BINERY_1. This corner is located in the
+  outer edge of aQuadrant (as defined by the CCD-related headers of aImage).
+
+  @error{set CPL_ERROR_NULL_INPUT\, return NULL,
+         aImage or its data and/or header extensions are NULL}
+ */
+/*---------------------------------------------------------------------------*/
+cpl_mask *
+muse_image_create_corner_mask(const muse_image *aImage, unsigned char aQuadrant,
+                              float aRadius)
+{
+  cpl_ensure(aImage && aImage->data && aImage->header, CPL_ERROR_NULL_INPUT,
+             NULL);
+
+  int i, j,
+      nx = cpl_image_get_size_x(aImage->data),
+      ny = cpl_image_get_size_y(aImage->data),
+      outnx = muse_pfits_get_out_output_x(aImage->header, aQuadrant),
+      outny = muse_pfits_get_out_output_y(aImage->header, aQuadrant);
+  cpl_msg_debug(__func__, "Origin: %d,%d", outnx, outny);
+
+  cpl_mask *mask = cpl_mask_new(nx, ny);
+  cpl_binary *m = cpl_mask_get_data(mask);
+  for (i = 1; i <= nx; i++) {
+    for (j = 1; j <= ny; j++) {
+      double r = sqrt((outnx - i) * (outnx - i) + (outny - j) * (outny - j));
+      if (r <= aRadius) {
+        m[(i-1) + (j-1)*nx] = CPL_BINARY_1;
+      }
+    } /* for j (columns) */
+  } /* for i (rows) */
+
+  return mask;
+} /* muse_image_create_corner_mask() */
+
 /**@}*/
