@@ -736,8 +736,14 @@ static void lm_lmdif(int m, int n, double *x, double *fvec, double ftol,
 /* outer: calculate the jacobian matrix. */
     *info = 0;
 #if 1
-    #pragma omp parallel for default(none)               /* as req. by Ralf */ \
-            shared(n, info, m, x, eps, evaluate, data, printout, nfev, iter,   \
+    // FIXME: Removed default(none) clause here, to make the code work with
+    //        gcc9 while keeping older versions of gcc working too without
+    //        resorting to conditional compilation. Having default(none) here
+    //        causes gcc9 to abort the build because of __func__ not being
+    //        specified in a data sharing clause. Adding a data sharing clause
+    //        makes older gcc versions choke.
+    #pragma omp parallel for                                                 \
+            shared(n, info, m, x, eps, evaluate, data, printout, nfev, iter, \
                    fjac, fvec)
 #endif
     for (j = 0; j < n; j++) {
