@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2017 European Southern Observatory
+ * Copyright (C) 2005-2018 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -92,6 +92,25 @@ muse_pfits_get_pipefile(const cpl_propertylist *aHeaders)
   return value;
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the ancestor of a file.
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or NULL on error
+
+  Queries FITS header for ESO.PRO.ANCESTOR
+ */
+/*----------------------------------------------------------------------------*/
+const char *
+muse_pfits_get_ancestor(const cpl_propertylist *aHeaders)
+{
+  cpl_ensure(aHeaders, CPL_ERROR_NULL_INPUT, NULL);
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const char *value = cpl_propertylist_get_string(aHeaders, "ESO PRO ANCESTOR");
+  cpl_errorstate_set(prestate);
+  return value;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    find out the DPR type
@@ -164,6 +183,274 @@ muse_pfits_get_pro_catg(const cpl_propertylist *aHeaders)
   return value;
 } /* muse_pfits_get_pro_catg() */
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the i-th raw file name.
+  @param    aHeaders       property list/headers to read from
+  @param    idx            raw file index
+  @return   the requested value or NULL on error
+
+  Queries FITS header for ESO.PRO.REC1.RAW<idx>.NAME
+ */
+/*----------------------------------------------------------------------------*/
+const char *
+muse_pfits_get_raw_filename(const cpl_propertylist *aHeaders, unsigned int idx)
+{
+  cpl_ensure(aHeaders, CPL_ERROR_NULL_INPUT, NULL);
+  char *key = cpl_sprintf("ESO PRO REC1 RAW%-u NAME", idx);
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const char *value = cpl_propertylist_get_string(aHeaders, key);
+  cpl_errorstate_set(prestate);
+  cpl_free(key);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the value of the flux calibration flag
+  @param    aHeaders       property list/headers to read from
+  @param    idx            recipe index
+  @return   the requested value or NULL on error
+
+  Queries FITS header for ESO.PRO.REC<idx>.PIPE.ID
+ */
+/*----------------------------------------------------------------------------*/
+const char *
+muse_pfits_get_pipe_id(const cpl_propertylist *aHeaders, unsigned int idx)
+{
+  cpl_ensure(aHeaders, CPL_ERROR_NULL_INPUT, NULL);
+  char *key = cpl_sprintf("ESO PRO REC%-u PIPE ID", idx);
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const char *value = cpl_propertylist_get_string(aHeaders, key);
+  cpl_errorstate_set(prestate);
+  cpl_free(key);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the ESO program identification
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or NULL on error
+
+  Queries FITS header for ESO.OBS.PROG.ID
+ */
+/*----------------------------------------------------------------------------*/
+const char *
+muse_pfits_get_progid(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const char *value = cpl_propertylist_get_string(aHeaders, "ESO OBS PROG ID");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), NULL);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the ESO observation target name
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or NULL on error
+
+  Queries FITS header for ESO.OBS.TARG.NAME
+ */
+/*----------------------------------------------------------------------------*/
+const char *
+muse_pfits_get_targname(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const char *value = cpl_propertylist_get_string(aHeaders, "ESO OBS TARG NAME");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), NULL);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the observation block id
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or -1 on error
+
+  Queries FITS header for ESO.OBS.ID
+ */
+/*----------------------------------------------------------------------------*/
+long
+muse_pfits_get_obsid(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const long value = cpl_propertylist_get_long(aHeaders, "ESO OBS ID");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), -1);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    query the high-order loop status
+  @param    aHeaders       property list/headers to read from
+  @return   CPL_TRUE or CPL_FALSE;
+            CPL_FALSE is also returned, if the header keyword does not exist
+
+  Queries FITS header ESO AOS HO LOOP ST.
+  CPL_TRUE signifies that AO was switched on.
+ */
+/*----------------------------------------------------------------------------*/
+cpl_boolean
+muse_pfits_get_ho_loop(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const cpl_boolean value = cpl_propertylist_get_bool(aHeaders,
+                                                      "ESO AOS HO LOOP ST") == 0
+                          ? CPL_FALSE : CPL_TRUE;
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), CPL_FALSE);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    query the WFM tit-tilt loop status
+  @param    aHeaders       property list/headers to read from
+  @return   CPL_TRUE or CPL_FALSE;
+            CPL_FALSE is also returned, if the header keyword does not exist
+
+  Queries FITS header ESO AOS TT LOOP ST.
+  CPL_TRUE signifies that WFM AO was using the tip-tilt correction. For tip-tilt
+  free operation, this will be CPL_FALSE.
+ */
+/*----------------------------------------------------------------------------*/
+cpl_boolean
+muse_pfits_get_tt_loop(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const cpl_boolean value = cpl_propertylist_get_bool(aHeaders,
+                                                      "ESO AOS TT LOOP ST") == 0
+                          ? CPL_FALSE : CPL_TRUE;
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), CPL_FALSE);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    query the NFM IRLOS loop status
+  @param    aHeaders       property list/headers to read from
+  @return   CPL_TRUE or CPL_FALSE;
+            CPL_FALSE is also returned, if the header keyword does not exist
+
+  Queries FITS header ESO AOS IR LOOP ST.
+  CPL_TRUE signifies that NFM AO was using the tip-tilt correction as measured
+  by the IRLOS sensor. For tip-tilt free operation, this will be CPL_FALSE.
+ */
+/*----------------------------------------------------------------------------*/
+cpl_boolean
+muse_pfits_get_ir_loop(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const cpl_boolean value = cpl_propertylist_get_bool(aHeaders,
+                                                      "ESO AOS IR LOOP ST") == 0
+                          ? CPL_FALSE : CPL_TRUE;
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), CPL_FALSE);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    query the AO mirror name
+  @param    aHeaders       property list/headers to read from
+  @param    aMirror        the mirror number (the "i" of the header keyword)
+  @return   the mirror name or NULL
+
+  Queries FITS header ESO.INS.AO.MIRRi.NAME
+
+  Use aMirror==2 to query that the GALACSI AO instrument mode is the correct
+  one (it should be either "WFM" or "NFM").
+  Use aMirror==3 to check that the GALACSI dichroic is out of the light path;
+  then the resulting string should be "OUT".
+ */
+/*----------------------------------------------------------------------------*/
+const char *
+muse_pfits_get_ao_mirror_name(const cpl_propertylist *aHeaders,
+                              unsigned int aMirror)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  char keyword[KEYWORD_LENGTH];
+  snprintf(keyword, KEYWORD_LENGTH, "ESO INS AO MIRR%u NAME", aMirror);
+  const char *value = cpl_propertylist_get_string(aHeaders, keyword);
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), NULL);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    query the AO optical element name
+  @param    aHeaders       property list/headers to read from
+  @param    aElement       the element number (the "i" of the header keyword)
+  @return   the mirror name or NULL
+
+  Queries FITS header  ESO.INS.AO.OPTIi.NAME
+
+  Use aElement==3 to check that the GALACSI dichroic is out of the light path;
+  then the resulting string should be "OUT".
+ */
+/*----------------------------------------------------------------------------*/
+const char *muse_pfits_get_ao_opti_name(const cpl_propertylist *aHeaders,
+                                        unsigned int aElement)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  char keyword[KEYWORD_LENGTH];
+  snprintf(keyword, KEYWORD_LENGTH, "ESO INS AO OPTI%u NAME", aElement);
+  const char *value = cpl_propertylist_get_string(aHeaders, keyword);
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), NULL);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the observation mode
+  @param    aHeaders       property list/headers to read from
+  @return   The observing mode, on error this is MUSE_MODE_WFM_NONAO_N as the
+            default MUSE setup.
+
+  Queries header ESO INS MODE
+  Uses @ref muse_pfits_get_insmode() to query the string of the header.
+ */
+/*----------------------------------------------------------------------------*/
+muse_ins_mode
+muse_pfits_get_mode(const cpl_propertylist *aHeaders)
+{
+  const char *value = muse_pfits_get_insmode(aHeaders);
+  cpl_ensure(value, cpl_error_get_code(), MUSE_MODE_WFM_NONAO_N);
+  if (!strncmp(value, "NFM", 3)) {
+    return MUSE_MODE_NFM_AO_N;
+  }
+  if (!strncmp(value, "WFM-AO-N", 8)) {
+    return MUSE_MODE_WFM_AO_N;
+  }
+  if (!strncmp(value, "WFM-AO-E", 8)) {
+    return MUSE_MODE_WFM_AO_E;
+  }
+  if (!strncmp(value, "WFM-NOAO-N", 10) || !strncmp(value, "WFM-NONAO-N", 11)) {
+    return MUSE_MODE_WFM_NONAO_N;
+  }
+  return MUSE_MODE_WFM_NONAO_E;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the observation mode
+  @param    aHeaders       property list/headers to read from
+  @return   The observing mode as string or NULL on error
+
+  Queries header ESO INS MODE
+  The difference to @ref muse_pfits_get_mode() is that this function returns
+  the string content of the header, not the corresponding mode enum.
+ */
+/*----------------------------------------------------------------------------*/
+const char *
+muse_pfits_get_insmode(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const char *value = cpl_propertylist_get_string(aHeaders, "ESO INS MODE");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), NULL);
+  return value;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Find out the whether this header related to a certain IFU.
@@ -192,6 +479,78 @@ muse_pfits_has_ifu(const cpl_propertylist *aHeaders, unsigned char aIFU)
   return chan == aIFU;
 } /* muse_pfits_has_ifu() */
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the MUSE derotator mode
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or 0.0 on error
+
+  Queries FITS header ESO INS DROT MODE
+ */
+/*----------------------------------------------------------------------------*/
+const char *
+muse_pfits_get_drot_mode(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const char *value = cpl_propertylist_get_string(aHeaders, "ESO INS DROT MODE");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), NULL);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the MUSE derotator position angle (in degrees)
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or 0.0 on error
+
+  Queries FITS header ESO INS DROT POSANG
+ */
+/*----------------------------------------------------------------------------*/
+double
+muse_pfits_get_drot_posang(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const double value = cpl_propertylist_get_double(aHeaders, "ESO INS DROT POSANG");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the MUSE derotator rotation at exposure start (in degrees)
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or 0.0 on error
+
+  Queries FITS header ESO INS DROT START
+ */
+/*----------------------------------------------------------------------------*/
+double
+muse_pfits_get_drot_start(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const double value = cpl_propertylist_get_double(aHeaders, "ESO INS DROT START");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the MUSE derotator rotation at exposure end (in degrees)
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or 0.0 on error
+
+  Queries FITS header ESO INS DROT END
+ */
+/*----------------------------------------------------------------------------*/
+double
+muse_pfits_get_drot_end(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const double value = cpl_propertylist_get_double(aHeaders, "ESO INS DROT END");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
+  return value;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    find out the extension name
@@ -993,116 +1352,6 @@ muse_pfits_get_airmass_end(const cpl_propertylist *aHeaders)
   return value;
 }
 
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    find out the ambient temperature (in degrees Celsius)
-  @param    aHeaders       property list/headers to read from
-  @return   the requested value or 0.0 on error
-
-  Queries FITS header ESO TEL AMBI TEMP
- */
-/*----------------------------------------------------------------------------*/
-double
-muse_pfits_get_temp(const cpl_propertylist *aHeaders)
-{
-  cpl_errorstate prestate = cpl_errorstate_get();
-  const double value = cpl_propertylist_get_double(aHeaders, "ESO TEL AMBI TEMP");
-  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
-  return value;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    find out the relavtive humidity (in %)
-  @param    aHeaders       property list/headers to read from
-  @return   the requested value or 0.0 on error
-
-  Queries FITS header ESO TEL AMBI RHUM
- */
-/*----------------------------------------------------------------------------*/
-double
-muse_pfits_get_rhum(const cpl_propertylist *aHeaders)
-{
-  cpl_errorstate prestate = cpl_errorstate_get();
-  const double value = cpl_propertylist_get_double(aHeaders, "ESO TEL AMBI RHUM");
-  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
-  return value;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    find out the ambient pressure at start of exposure (in mbar)
-  @param    aHeaders       property list/headers to read from
-  @return   the requested value or 0.0 on error
-
-  Queries FITS header ESO TEL AMBI PRES START
- */
-/*----------------------------------------------------------------------------*/
-double
-muse_pfits_get_pres_start(const cpl_propertylist *aHeaders)
-{
-  cpl_errorstate prestate = cpl_errorstate_get();
-  const double value = cpl_propertylist_get_double(aHeaders, "ESO TEL AMBI PRES START");
-  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
-  return value;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    find out the ambient pressure at end of exposure (in mbar)
-  @param    aHeaders       property list/headers to read from
-  @return   the requested value or 0.0 on error
-
-  Queries FITS header ESO TEL AMBI PRES END
- */
-/*----------------------------------------------------------------------------*/
-double
-muse_pfits_get_pres_end(const cpl_propertylist *aHeaders)
-{
-  cpl_errorstate prestate = cpl_errorstate_get();
-  const double value = cpl_propertylist_get_double(aHeaders, "ESO TEL AMBI PRES END");
-  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
-  return value;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    find out the ambient seeing at start of exposure (in arcsec)
-  @param    aHeaders       property list/headers to read from
-  @return   the requested value or 0.0 on error
-
-  Queries FITS header ESO TEL AMBI FWHM START
- */
-/*----------------------------------------------------------------------------*/
-double
-muse_pfits_get_fwhm_start(const cpl_propertylist *aHeaders)
-{
-  cpl_errorstate prestate = cpl_errorstate_get();
-  const double value = cpl_propertylist_get_double(aHeaders, "ESO TEL AMBI FWHM START");
-  cpl_ensure(cpl_errorstate_is_equal(prestate) && value > 0.,
-             cpl_error_get_code(), 0.0);
-  return value;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    find out the ambient seeing at end of exposure (in arcsec)
-  @param    aHeaders       property list/headers to read from
-  @return   the requested value or 0.0 on error
-
-  Queries FITS header ESO TEL AMBI FWHM END
- */
-/*----------------------------------------------------------------------------*/
-double
-muse_pfits_get_fwhm_end(const cpl_propertylist *aHeaders)
-{
-  cpl_errorstate prestate = cpl_errorstate_get();
-  const double value = cpl_propertylist_get_double(aHeaders, "ESO TEL AMBI FWHM END");
-  cpl_ensure(cpl_errorstate_is_equal(prestate) && value > 0.,
-             cpl_error_get_code(), 0.0);
-  return value;
-}
-
 /*----------------------------------------------------------------------------*/
 /**
   @brief    find out the altitude angle at start of the exposure (in degrees)
@@ -1267,124 +1516,111 @@ muse_pfits_get_ia_fwhmlin(const cpl_propertylist *aHeaders)
 
 /*----------------------------------------------------------------------------*/
 /**
-  @brief    find out the MUSE derotator mode
+  @brief    find out the ambient temperature (in degrees Celsius)
   @param    aHeaders       property list/headers to read from
   @return   the requested value or 0.0 on error
 
-  Queries FITS header ESO INS DROT MODE
- */
-/*----------------------------------------------------------------------------*/
-const char *
-muse_pfits_get_drot_mode(const cpl_propertylist *aHeaders)
-{
-  cpl_errorstate prestate = cpl_errorstate_get();
-  const char *value = cpl_propertylist_get_string(aHeaders, "ESO INS DROT MODE");
-  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), NULL);
-  return value;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    find out the MUSE derotator position angle (in degrees)
-  @param    aHeaders       property list/headers to read from
-  @return   the requested value or 0.0 on error
-
-  Queries FITS header ESO INS DROT POSANG
+  Queries FITS header ESO TEL AMBI TEMP
  */
 /*----------------------------------------------------------------------------*/
 double
-muse_pfits_get_drot_posang(const cpl_propertylist *aHeaders)
+muse_pfits_get_temp(const cpl_propertylist *aHeaders)
 {
   cpl_errorstate prestate = cpl_errorstate_get();
-  const double value = cpl_propertylist_get_double(aHeaders, "ESO INS DROT POSANG");
+  const double value = cpl_propertylist_get_double(aHeaders, "ESO TEL AMBI TEMP");
   cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
   return value;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
-  @brief    find out the MUSE derotator rotation at exposure start (in degrees)
+  @brief    find out the relavtive humidity (in %)
   @param    aHeaders       property list/headers to read from
   @return   the requested value or 0.0 on error
 
-  Queries FITS header ESO INS DROT START
+  Queries FITS header ESO TEL AMBI RHUM
  */
 /*----------------------------------------------------------------------------*/
 double
-muse_pfits_get_drot_start(const cpl_propertylist *aHeaders)
+muse_pfits_get_rhum(const cpl_propertylist *aHeaders)
 {
   cpl_errorstate prestate = cpl_errorstate_get();
-  const double value = cpl_propertylist_get_double(aHeaders, "ESO INS DROT START");
+  const double value = cpl_propertylist_get_double(aHeaders, "ESO TEL AMBI RHUM");
   cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
   return value;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
-  @brief    find out the MUSE derotator rotation at exposure end (in degrees)
+  @brief    find out the ambient pressure at start of exposure (in mbar)
   @param    aHeaders       property list/headers to read from
   @return   the requested value or 0.0 on error
 
-  Queries FITS header ESO INS DROT END
+  Queries FITS header ESO TEL AMBI PRES START
  */
 /*----------------------------------------------------------------------------*/
 double
-muse_pfits_get_drot_end(const cpl_propertylist *aHeaders)
+muse_pfits_get_pres_start(const cpl_propertylist *aHeaders)
 {
   cpl_errorstate prestate = cpl_errorstate_get();
-  const double value = cpl_propertylist_get_double(aHeaders, "ESO INS DROT END");
+  const double value = cpl_propertylist_get_double(aHeaders, "ESO TEL AMBI PRES START");
   cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
   return value;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
-  @brief    find out the observation mode
+  @brief    find out the ambient pressure at end of exposure (in mbar)
   @param    aHeaders       property list/headers to read from
-  @return   The observing mode, on error this is MUSE_MODE_WFM_NONAO_N as the
-            default MUSE setup.
+  @return   the requested value or 0.0 on error
 
-  Queries header ESO INS MODE
-  Uses @ref muse_pfits_get_insmode() to query the string of the header.
+  Queries FITS header ESO TEL AMBI PRES END
  */
 /*----------------------------------------------------------------------------*/
-muse_ins_mode
-muse_pfits_get_mode(const cpl_propertylist *aHeaders)
+double
+muse_pfits_get_pres_end(const cpl_propertylist *aHeaders)
 {
-  const char *value = muse_pfits_get_insmode(aHeaders);
-  cpl_ensure(value, cpl_error_get_code(), MUSE_MODE_WFM_NONAO_N);
-  if (!strncmp(value, "NFM", 3)) {
-    return MUSE_MODE_NFM_AO_N;
-  }
-  if (!strncmp(value, "WFM-AO-N", 8)) {
-    return MUSE_MODE_WFM_AO_N;
-  }
-  if (!strncmp(value, "WFM-AO-E", 8)) {
-    return MUSE_MODE_WFM_AO_E;
-  }
-  if (!strncmp(value, "WFM-NOAO-N", 10) || !strncmp(value, "WFM-NONAO-N", 11)) {
-    return MUSE_MODE_WFM_NONAO_N;
-  }
-  return MUSE_MODE_WFM_NONAO_E;
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const double value = cpl_propertylist_get_double(aHeaders, "ESO TEL AMBI PRES END");
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
+  return value;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
-  @brief    find out the observation mode
+  @brief    find out the ambient seeing at start of exposure (in arcsec)
   @param    aHeaders       property list/headers to read from
-  @return   The observing mode as string or NULL on error
+  @return   the requested value or 0.0 on error
 
-  Queries header ESO INS MODE
-  The difference to @ref muse_pfits_get_mode() is that this function returns
-  the string content of the header, not the corresponding mode enum.
+  Queries FITS header ESO TEL AMBI FWHM START
  */
 /*----------------------------------------------------------------------------*/
-const char *
-muse_pfits_get_insmode(const cpl_propertylist *aHeaders)
+double
+muse_pfits_get_fwhm_start(const cpl_propertylist *aHeaders)
 {
   cpl_errorstate prestate = cpl_errorstate_get();
-  const char *value = cpl_propertylist_get_string(aHeaders, "ESO INS MODE");
-  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), NULL);
+  const double value = cpl_propertylist_get_double(aHeaders, "ESO TEL AMBI FWHM START");
+  cpl_ensure(cpl_errorstate_is_equal(prestate) && value > 0.,
+             cpl_error_get_code(), 0.0);
+  return value;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    find out the ambient seeing at end of exposure (in arcsec)
+  @param    aHeaders       property list/headers to read from
+  @return   the requested value or 0.0 on error
+
+  Queries FITS header ESO TEL AMBI FWHM END
+ */
+/*----------------------------------------------------------------------------*/
+double
+muse_pfits_get_fwhm_end(const cpl_propertylist *aHeaders)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  const double value = cpl_propertylist_get_double(aHeaders, "ESO TEL AMBI FWHM END");
+  cpl_ensure(cpl_errorstate_is_equal(prestate) && value > 0.,
+             cpl_error_get_code(), 0.0);
   return value;
 }
 
@@ -1521,6 +1757,9 @@ muse_pfits_get_lamp_name(const cpl_propertylist *aHeaders, int aLamp)
   @return   1 for lamp status on (T) or 0 for off (F) or error
 
   Queries FITS header ESO INS LAMPi ST
+  @note This just checks the status of the lamp at the end of the exposure.
+        Use the function @ref muse_pfits_get_lamp_ontime() to check, if the
+        lamp was on at all during the given exposure.
  */
 /*----------------------------------------------------------------------------*/
 int
@@ -1534,6 +1773,30 @@ muse_pfits_get_lamp_status(const cpl_propertylist *aHeaders, int aLamp)
   return value;
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    query the time that a lamp was on during an exposure
+  @param    aHeaders       property list/headers to read from
+  @param    aLamp          number of the lamp to query
+  @return   the time in seconds that the lamp was on or 0.0 on error.
+
+  Queries FITS header ESO SEQ LAMPi ONTIME
+
+  This is useful to check, if a lamp was on at all during an exposure, even if
+  @ref muse_pfits_get_lamp_status() returns false.
+ */
+/*----------------------------------------------------------------------------*/
+double
+muse_pfits_get_lamp_ontime(const cpl_propertylist *aHeaders, int aLamp)
+{
+  cpl_errorstate prestate = cpl_errorstate_get();
+  char keyword[KEYWORD_LENGTH];
+  snprintf(keyword, KEYWORD_LENGTH, "ESO SEQ LAMP%d ONTIME", aLamp);
+  const double value = cpl_propertylist_get_double(aHeaders, keyword);
+  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), 0.0);
+  return value;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    query the name of one shutter
@@ -1628,121 +1891,4 @@ muse_pfits_get_pospos(const cpl_propertylist *aHeaders, unsigned short aEncoder)
   return value;
 }
 
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    find out the observation block id
-  @param    aHeaders       property list/headers to read from
-  @return   the requested value or -1 on error
-
-  Queries FITS header for ESO.OBS.ID
- */
-/*----------------------------------------------------------------------------*/
-long
-muse_pfits_get_obsid(const cpl_propertylist *aHeaders)
-{
-  cpl_errorstate prestate = cpl_errorstate_get();
-  const long value = cpl_propertylist_get_long(aHeaders, "ESO OBS ID");
-  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), -1);
-  return value;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    find out the ESO program identification
-  @param    aHeaders       property list/headers to read from
-  @return   the requested value or NULL on error
-
-  Queries FITS header for ESO.OBS.PROG.ID
- */
-/*----------------------------------------------------------------------------*/
-const char *
-muse_pfits_get_progid(const cpl_propertylist *aHeaders)
-{
-  cpl_errorstate prestate = cpl_errorstate_get();
-  const char *value = cpl_propertylist_get_string(aHeaders, "ESO OBS PROG ID");
-  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), NULL);
-  return value;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    find out the ESO observation target name
-  @param    aHeaders       property list/headers to read from
-  @return   the requested value or NULL on error
-
-  Queries FITS header for ESO.OBS.TARG.NAME
- */
-/*----------------------------------------------------------------------------*/
-const char *
-muse_pfits_get_targname(const cpl_propertylist *aHeaders)
-{
-  cpl_errorstate prestate = cpl_errorstate_get();
-  const char *value = cpl_propertylist_get_string(aHeaders, "ESO OBS TARG NAME");
-  cpl_ensure(cpl_errorstate_is_equal(prestate), cpl_error_get_code(), NULL);
-  return value;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    find out the value of the flux calibration flag
-  @param    aHeaders       property list/headers to read from
-  @param    idx            recipe index
-  @return   the requested value or NULL on error
-
-  Queries FITS header for ESO.PRO.REC<idx>.PIPE.ID
- */
-/*----------------------------------------------------------------------------*/
-const char *
-muse_pfits_get_pipe_id(const cpl_propertylist *aHeaders, unsigned int idx)
-{
-  cpl_ensure(aHeaders, CPL_ERROR_NULL_INPUT, NULL);
-  char *key = cpl_sprintf("ESO PRO REC%-u PIPE ID", idx);
-  cpl_errorstate prestate = cpl_errorstate_get();
-  const char *value = cpl_propertylist_get_string(aHeaders, key);
-  cpl_errorstate_set(prestate);
-  cpl_free(key);
-  return value;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    find out the i-th raw file name.
-  @param    aHeaders       property list/headers to read from
-  @param    idx            raw file index
-  @return   the requested value or NULL on error
-
-  Queries FITS header for ESO.PRO.REC1.RAW<idx>.NAME
- */
-/*----------------------------------------------------------------------------*/
-const char *
-muse_pfits_get_raw_filename(const cpl_propertylist *aHeaders, unsigned int idx)
-{
-  cpl_ensure(aHeaders, CPL_ERROR_NULL_INPUT, NULL);
-  char *key = cpl_sprintf("ESO PRO REC1 RAW%-u NAME", idx);
-  cpl_errorstate prestate = cpl_errorstate_get();
-  const char *value = cpl_propertylist_get_string(aHeaders, key);
-  cpl_errorstate_set(prestate);
-  cpl_free(key);
-  return value;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief    find out the ancestor of a file.
-  @param    aHeaders       property list/headers to read from
-  @return   the requested value or NULL on error
-
-  Queries FITS header for ESO.PRO.ANCESTOR
- */
-/*----------------------------------------------------------------------------*/
-const char *
-muse_pfits_get_ancestor(const cpl_propertylist *aHeaders)
-{
-  cpl_ensure(aHeaders, CPL_ERROR_NULL_INPUT, NULL);
-  cpl_errorstate prestate = cpl_errorstate_get();
-  const char *value = cpl_propertylist_get_string(aHeaders, "ESO PRO ANCESTOR");
-  cpl_errorstate_set(prestate);
-  return value;
-}
-
 /**@}*/
