@@ -70,7 +70,7 @@
   @error{return CPL_ERROR_NULL_INPUT,
          if aImage or its data or header component are NULL}
   @error{return CPL_ERROR_BAD_FILE_FORMAT,
-         region sizes are not equal for all quadrants}
+         section sizes are not equal for all quadrants}
   @error{return CPL_ERROR_ILLEGAL_INPUT,
          FITS headers are missing required keywords or aIgnore as large as the horizontal overscan width or larger}
  */
@@ -80,7 +80,7 @@ muse_quadrants_overscan_stats(muse_image *aImage, const char *aRejection,
                               unsigned int aIgnore)
 {
   cpl_ensure_code(aImage && aImage->data && aImage->header, CPL_ERROR_NULL_INPUT);
-  /* ensure that the region sizes are the same for all quadrants */
+  /* ensure that the section sizes are the same for all quadrants */
   cpl_ensure_code(muse_quadrants_verify(aImage->header),
                   CPL_ERROR_BAD_FILE_FORMAT);
   /* Check that the overscan windows make sense for all quadrants; *
@@ -129,7 +129,7 @@ muse_quadrants_overscan_stats(muse_image *aImage, const char *aRejection,
         }
       }
     }
-    /* check that box sizes in both directions stay inside overscan regions */
+    /* check that box sizes in both directions stay inside overscan region */
     if (dcrbox2 > overx || dcrbox2 > overy ||
         dcrbox1 > outnx || dcrbox1 > outny ||
         dcrnpass <= 0 || dcrthres <= 0.) {
@@ -163,7 +163,7 @@ muse_quadrants_overscan_stats(muse_image *aImage, const char *aRejection,
   const float *data = cpl_image_get_data_float_const(aImage->data);
   const int *dq = cpl_image_get_data_int_const(aImage->dq);
 
-  /* get data region for each quadrant and compute the size of the output image */
+  /* get data section for each quadrant and compute the size of the output image */
   for (n = 1; n <= 4; n++) {
     /* get overscan regions and compute pixel numbers */
     cpl_size *w = muse_quadrants_overscan_get_window(aImage, n, aIgnore);
@@ -423,11 +423,11 @@ muse_quadrants_overscan_correct(muse_image *aImage, muse_image *aRefImage)
   This function modifies the input image (the data component) and the header,
   trying to remove possible large-scale vertical bias structure. The steps
   involved are:
-  - check all relevant header keywords related to the CCD regions
+  - check all relevant header keywords related to the CCD sections
   - loop through the four quadrants:
     . set each valid input pixel and vertical position in the vertical overscan
-      region into CPL structures
-    . exclude pixels closer to the illuminated CCD region than aIgnore pixels
+      section into CPL structures
+    . exclude pixels closer to the illuminated CCD sections than aIgnore pixels
     . use the CPL structures to fit 1D polynomials using subsequently higher
       polynomial order
     . aSigma is the sigma-level used for iterative rejection of points in each
@@ -469,7 +469,7 @@ muse_quadrants_overscan_polyfit_vertical(muse_image *aImage, unsigned aIgnore,
   cpl_ensure_code(muse_quadrants_verify(aImage->header),
                   CPL_ERROR_BAD_FILE_FORMAT);
 
-  /* get output sizes for data regions, and pre- and overscans */
+  /* get output sizes for data sections, and pre- and overscans */
   int nx = cpl_image_get_size_x(aImage->data),
       binx = muse_pfits_get_binx(aImage->header),
       biny = muse_pfits_get_biny(aImage->header),
@@ -690,7 +690,7 @@ muse_quadrants_overscan_get_window(const muse_image *aImage,
   cpl_ensure(aImage && aImage->data && aImage->header, CPL_ERROR_NULL_INPUT,
              NULL);
   cpl_ensure(aQuadrant >= 1 && aQuadrant <= 4, CPL_ERROR_ILLEGAL_INPUT, NULL);
-  /* get output sizes for data regions, and pre- and overscans */
+  /* get output sizes for data sections, and pre- and overscans */
   cpl_errorstate state = cpl_errorstate_get();
   int binx = muse_pfits_get_binx(aImage->header),
       biny = muse_pfits_get_biny(aImage->header),
@@ -781,13 +781,13 @@ muse_quadrants_trim_image(muse_image *aImage)
 
   /* output image size */
   int xout = 0, yout = 0;
-  /* arrays of input data region sizes, and output port locations */
+  /* arrays of input data section sizes, and output port locations */
   int nx[4], ny[4], portx[4], porty[4];
   /* binning to take into account */
   int binx = muse_pfits_get_binx(aImage->header),
       biny = muse_pfits_get_biny(aImage->header);
 
-  /* get data region for each quadrant and compute the size of the output image */
+  /* get data section for each quadrant and compute the size of the output image */
   unsigned char n; /* quadrant counter */
   for (n = 1; n <= 4; n++) {
     nx[n-1] = muse_pfits_get_out_nx(aImage->header, n) / binx;
@@ -836,10 +836,10 @@ muse_quadrants_trim_image(muse_image *aImage)
   }
   cpl_ensure(xout > 0 && yout > 0, CPL_ERROR_ILLEGAL_INPUT, NULL);
 
-  /* verify that all the data regions have the same size */
+  /* verify that all the data sections have the same size */
   for (n = 1; n < 4; n++) {
     if (nx[n] != nx[0] || ny[n] != ny[0]) {
-      cpl_msg_error(__func__, "Data region of quadrant %d is different from "
+      cpl_msg_error(__func__, "Data section of quadrant %d is different from "
                     "quadrant 1!", n+1);
       cpl_error_set(__func__, CPL_ERROR_INCOMPATIBLE_INPUT);
       return NULL;
@@ -864,14 +864,14 @@ muse_quadrants_trim_image(muse_image *aImage)
                                 "^ESO DET OUT.*PRSC|^ESO DET OUT.*OVSC",
                                 0);
 
-  /* copy data regions (of all three extensions, if they exist) *
+  /* copy data sections (of all three extensions, if they exist) *
    * and for all quadrants into the output image                */
   for (n = 1; n <= 4; n++) {
     /* read properties from input header */
     int x_prescan = muse_pfits_get_out_prescan_x(aImage->header, n) / binx,
         y_prescan = muse_pfits_get_out_prescan_y(aImage->header, n) / biny;
 
-    /* determine the data regions to extract and the target output positions */
+    /* determine the data sections to extract and the target output positions */
     int x1 = 0, x2 = 0, y1 = 0, y2= 0,
         xtarget = 0, ytarget = 0;
 
