@@ -173,6 +173,148 @@ muse_quality_dark_badpix(muse_image *aDark, double aSigmaLo, double aSigmaHi)
   return nlo + nhi;
 } /* muse_quality_dark_badpix() */
 
+/*---------------------------------------------------------------------------*/
+/**
+  @brief    Find more bad pixels in residuals of a master dark.
+  @param    aDarkRes   MUSE dark image to process
+  @param    aSigma     sigma for low and high cuts (in terms of median deviation)
+  @param    aIter      number of iterations to search for bad neightbors
+  @return   number of bad pixels found or a negative value on error
+
+  This function is thought to operate on master dark images which are
+  bias-subtracted and where the background was already pre-subtracted using some
+  model. This allows this function to use stricter sigma-clipping levels than
+  possible without the subtraction, as used in @ref muse_quality_dark_badpix().
+
+  Determine outliers that deviate too more than aSigma * median-deviation across
+  the whole image. In a first step, they are only marked as bad, if there is at
+  least one neighboring pixel that is above the limit as well.  The low pixels
+  are flagged EURO3D_DEADPIXEL, the high (hot) pixels as EURO3D_HOTPIXEL in the
+  dq extension of aDarkRes. Any existing bad-pixel mask of the cpl_images and
+  the stat extension (containing the data variance) are ignored by this
+  function, but pixels that are already flagged in the dq extension are not
+  touched again (and not counted).
+
+  If aIter is positive, further bad neightbors (strictly in x and y direction)
+  are searched iteratively: if all four neightbors of a pixel are deviant, mark
+  it as bad as well.
+
+  Note that incoming bad pixels (marked as bad in the incoming bad-pixel mask of
+  the cpl_image of the data component) are excluded from the statistics, but are
+  otherwise treated like all other pixels.
+
+  @error{set CPL_ERROR_NULL_INPUT and return -1,
+         aDarkRes or its data and/or dq extensions are NULL}
+  @error{set CPL_ERROR_ILLEGAL_INPUT and return -2,
+         the data buffers of the data or dq components of aDarkRes could not be accessed}
+ */
+/*---------------------------------------------------------------------------*/
+int
+muse_quality_dark_refine_badpix(muse_image *aDarkRes, double aSigma,
+                                unsigned short aIter)
+{
+  cpl_ensure(aDarkRes, CPL_ERROR_NULL_INPUT, -1);
+
+  cpl_stats_mode smode = CPL_STATS_MEDIAN | CPL_STATS_MEDIAN_DEV;
+  cpl_stats *s = cpl_stats_new_from_image(aDarkRes->data, smode);
+  double median = cpl_stats_get_median(s),
+         mdev = cpl_stats_get_median_dev(s),
+         limit = aSigma * mdev;
+  cpl_msg_debug(__func__, "bad pixel limit: %f (%f +/- %f)", limit, median,
+                mdev);
+#if 0
+  cpl_stats_dump(s, smode, stdout);
+  fflush(stdout);
+#endif
+  cpl_stats_delete(s);
+
+  const float *res = cpl_image_get_data_float_const(aDarkRes->data);
+  int *dq = cpl_image_get_data_int(aDarkRes->dq);
+  cpl_ensure(res && dq, CPL_ERROR_ILLEGAL_INPUT, -2);
+  int i, j, n = 0,
+      nx = cpl_image_get_size_x(aDarkRes->data),
+      ny = cpl_image_get_size_y(aDarkRes->data);
+  for (i = 0; i < nx; i++) {
+    for (j = 0; j < ny; j++) {
+      if (dq[i + j*nx]) {
+        /* already a bad pixel, do nothing */
+        continue;
+      }
+      double r = res[i + j*nx];
+      if (fabs(r) > limit) {
+        /* check neighbors */
+        int nhi = 0;
+        /* left */
+        if (i > 0 && fabs(res[(i-1) + j*nx]) > limit) {
+          nhi++;
+        }
+        /* right */
+        if (i < (nx-1) && fabs(res[(i+1) + j*nx]) > limit) {
+          nhi++;
+        }
+        /* bottom */
+        if (j > 0 && fabs(res[i + (j-1)*nx]) > limit) {
+          nhi++;
+        }
+        /* top */
+        if (j < (ny-1) && fabs(res[i + (j+1)*nx]) > limit) {
+          nhi++;
+        }
+        /* if there are also at least two deviant neighbors, mark it as bad */
+        if (nhi > 1) {
+          if (r > 0) {
+            dq[i + j*nx] |= EURO3D_HOTPIXEL;
+          } else {
+            dq[i + j*nx] |= EURO3D_DARKPIXEL;
+          }
+          n++;
+        } /* if nhi */
+      }/* if r */
+    } /* for j (columns) */
+  } /* for i (rows) */
+  cpl_msg_debug(__func__, "%d new bad pixels after marking dubious cases", n);
+
+  /* next step: for insignificant residuals, check if all neighbors are bad */
+  unsigned short ni;
+  for (ni = 1; ni <= aIter; ni++) {
+    for (i = 0; i < nx; i++) {
+      for (j = 0; j < ny; j++) {
+        if (dq[i + j*nx]) {
+          /* already a bad pixel, do nothing */
+          continue;
+        }
+        /* count neighbors */
+        int nhi = 0;
+        if (i > 0 && fabs(res[(i-1) + j*nx]) > limit) {
+          nhi++;
+        }
+        if (i < (nx-1) && fabs(res[(i+1) + j*nx]) > limit) {
+          nhi++;
+        }
+        if (j > 0 && fabs(res[i + (j-1)*nx]) > limit) {
+          nhi++;
+        }
+        if (j < (ny-1) && fabs(res[i + (j+1)*nx]) > limit) {
+          nhi++;
+        }
+        /* if there are also deviant neighbors, mark it as bad */
+        if (nhi == 4) {
+          if (res[i + j*nx] > 0) {
+            dq[i + j*nx] |= EURO3D_HOTPIXEL;
+          } else {
+            dq[i + j*nx] |= EURO3D_DARKPIXEL;
+          }
+          n++;
+        } /* if nhi */
+      } /* for j (columns) */
+    } /* for i (rows) */
+    cpl_msg_debug(__func__, "%d new bad pixels after iteration %hu marking "
+                  "cases with bad neighbors", n, ni);
+  } /* for ni iterations */
+
+  return n;
+} /* muse_quality_dark_refine_badpix() */
+
 /*---------------------------------------------------------------------------*/
 /**
   @brief    Find bad columns (in a master bias).
