@@ -48,7 +48,7 @@
 /**@{*/
 
 /* corresponds to muse_rvcorrect_type, keep in sync with those values! */
-const char *muse_rvcorrect_typestring[] = {
+static const char *muse_rvcorrect_typestring[] = {
   "none",
   "bary",
   "helio",
@@ -56,6 +56,17 @@ const char *muse_rvcorrect_typestring[] = {
   "unknown"
 };
 
+/* Keyword values of the supported spectral reference frames, ordered         *
+ * according to the definition of the muse_rvcorrect_type enumeration values. */
+static const char *const
+muse_specsys_id[] = {
+ "TOPOCENT",
+ "BARYCENT",
+ "HELIOCEN",
+ "GEOCENTR",
+ "UNKNOWN"
+};
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief   Select type of radial velocity correction to be done from type string.
@@ -121,58 +132,67 @@ muse_rvcorrect(muse_pixtable *aPixtable, muse_rvcorrect_type aType)
 {
   cpl_ensure_code(aPixtable && aPixtable->table && aPixtable->header,
                   CPL_ERROR_NULL_INPUT);
+  const char *specsys = NULL;
   if (aType == MUSE_RVCORRECT_NONE) {
-    return CPL_ERROR_NONE;
-  }
+    specsys = muse_specsys_id[MUSE_RVCORRECT_NONE];
+  } else {
+    if (cpl_propertylist_has(aPixtable->header, MUSE_HDR_PT_RVCORR) &&
+        fabs(cpl_propertylist_get_double(aPixtable->header, MUSE_HDR_PT_RVCORR)) > 0.) {
+      cpl_msg_info(__func__, "pixel table already corrected: skipping radial "
+          "velocity correction");
+      return CPL_ERROR_NONE;
+    }
 
-  if (cpl_propertylist_has(aPixtable->header, MUSE_HDR_PT_RVCORR) &&
-      fabs(cpl_propertylist_get_double(aPixtable->header, MUSE_HDR_PT_RVCORR)) > 0.) {
-    cpl_msg_info(__func__, "pixel table already corrected: skipping radial "
-                 "velocity correction");
-    return CPL_ERROR_NONE;
-  }
+    cpl_errorstate state = cpl_errorstate_get();
+    muse_astro_rvcorr rvcorr = muse_astro_rvcorr_compute(aPixtable->header);
+    if (!cpl_errorstate_is_equal(state)) {
+      return cpl_error_set_message(__func__, cpl_error_get_code(), "Computing "
+                                   "radial velocity correction failed: %s",
+                                   cpl_error_get_message());
+    }
+    double rvoffset = 0.;
+    if (aType == MUSE_RVCORRECT_BARY) {
+      rvoffset = rvcorr.bary;
+      specsys = muse_specsys_id[MUSE_RVCORRECT_BARY];
+    } else if (aType == MUSE_RVCORRECT_HELIO) {
+      rvoffset = rvcorr.helio;
+      specsys = muse_specsys_id[MUSE_RVCORRECT_HELIO];
+    } else if (aType == MUSE_RVCORRECT_GEO) {
+      rvoffset = rvcorr.geo;
+      specsys = muse_specsys_id[MUSE_RVCORRECT_GEO];
+    } else { /* unknown type */
+      return cpl_error_set_message(__func__, CPL_ERROR_ILLEGAL_INPUT, "Unknown "
+                                   "type of radial velocity correction, no "
+                                   "correction performed!");
+    }
+    cpl_msg_info(__func__, "Correcting data for %scentric radial velocity of %.2f"
+                 " km/s", muse_rvcorrect_typestring[aType], rvoffset);
 
-  cpl_errorstate state = cpl_errorstate_get();
-  muse_astro_rvcorr rvcorr = muse_astro_rvcorr_compute(aPixtable->header);
-  if (!cpl_errorstate_is_equal(state)) {
-    return cpl_error_set_message(__func__, cpl_error_get_code(), "Computing "
-                                 "radial velocity correction failed: %s",
-                                 cpl_error_get_message());
-  }
-  double rvoffset = 0.;
-  if (aType == MUSE_RVCORRECT_BARY) {
-    rvoffset = rvcorr.bary;
-  } else if (aType == MUSE_RVCORRECT_HELIO) {
-    rvoffset = rvcorr.helio;
-  } else if (aType == MUSE_RVCORRECT_GEO) {
-    rvoffset = rvcorr.geo;
-  } else { /* unknown type */
-    return cpl_error_set_message(__func__, CPL_ERROR_ILLEGAL_INPUT, "Unknown "
-                                 "type of radial velocity correction, no "
-                                 "correction performed!");
-  }
-  cpl_msg_info(__func__, "Correcting data for %scentric radial velocity of %.2f"
-               " km/s", muse_rvcorrect_typestring[aType], rvoffset);
+    /* apply the velocity offset to all pixel table rows */
+    const double cinv = 1000. / CPL_PHYS_C; /* inverse speed of light in km/s */
+    float *lbda = cpl_table_get_data_float(aPixtable->table, MUSE_PIXTABLE_LAMBDA);
+    cpl_size irow, nrow = muse_pixtable_get_nrow(aPixtable);
+    #pragma omp parallel for default(none)                                       \
+        shared(lbda, nrow, rvoffset)                                             \
+        firstprivate(cinv)
+    for (irow = 0; irow < nrow; irow++) {
+      /* use the relativistic velocity correction formula, see e.g. IRAF dopcor */
+      lbda[irow] *= sqrt((1. + rvoffset * cinv) / (1. - rvoffset * cinv));
+    } /* for irow (all pixel table rows) */
 
-  /* apply the velocity offset to all pixel table rows */
-  const double cinv = 1000. / CPL_PHYS_C; /* inverse speed of light in km/s */
-  float *lbda = cpl_table_get_data_float(aPixtable->table, MUSE_PIXTABLE_LAMBDA);
-  cpl_size irow, nrow = muse_pixtable_get_nrow(aPixtable);
-  #pragma omp parallel for default(none)                                       \
-          shared(lbda, nrow, rvoffset)                                         \
-          firstprivate(cinv)
-  for (irow = 0; irow < nrow; irow++) {
-    /* use the relativistic velocity correction formula, see e.g. IRAF dopcor */
-    lbda[irow] *= sqrt((1. + rvoffset * cinv) / (1. - rvoffset * cinv));
-  } /* for irow (all pixel table rows) */
-
-  /* add the header to the pixel table to show that the correction was done */
-  cpl_propertylist_update_double(aPixtable->header, MUSE_HDR_PT_RVCORR,
-                                 rvoffset);
-  char *comment = cpl_sprintf(MUSE_HDR_PT_RVCORR_C,
-                              muse_rvcorrect_typestring[aType]);
-  cpl_propertylist_set_comment(aPixtable->header, MUSE_HDR_PT_RVCORR, comment);
-  cpl_free(comment);
+    /* add the header to the pixel table to show that the correction was done */
+    cpl_propertylist_update_double(aPixtable->header, MUSE_HDR_PT_RVCORR,
+                                   rvoffset);
+    char *comment = cpl_sprintf(MUSE_HDR_PT_RVCORR_C,
+                                muse_rvcorrect_typestring[aType]);
+    cpl_propertylist_set_comment(aPixtable->header, MUSE_HDR_PT_RVCORR, comment);
+    cpl_free(comment);
+  }
+  /* Set the spectral reference frame in the pixeltable header. This is also *
+   * needed if no correction was actually applied                            */
+  cpl_propertylist_update_string(aPixtable->header, "SPECSYS", specsys);
+  cpl_propertylist_set_comment(aPixtable->header, "SPECSYS",
+                               "Spectral reference frame");
   return CPL_ERROR_NONE;
 } /* muse_rvcorrect() */
 
