@@ -103,53 +103,60 @@ muse_sky_spectrum_from_cube(muse_datacube *aCube, const cpl_mask *aMask) {
 /*----------------------------------------------------------------------------*/
 /**
   @brief  Select spaxels to be considered as sky.
-  @param  aImage      MUSE (white-light) image of the field of view.
-  @param  aMinFraction Lowest fraction of spectra to ignore
-  @param  aFraction   Lowest fraction of remaining spectra to select
-  @param  aQCPrefix   prefix for the QC keywords
-  @return A newly created bitmask for sky spaxels.
+  @param  aImage         MUSE (white-light) image of the field of view.
+  @param  aMinFraction   Lowest fraction of spectra to ignore.
+  @param  aFraction      Lowest fraction of remaining spectra to select.
+  @param  aQCPrefix      Prefix for the QC keywords.
+  @return A newly created bitmask for sky spaxels or NULL on error.
 
   Use thresholding to create a mask of sky regions.
-  The output mask contains a QC parameter aQCPrefix" THRESHOLD" in its header
-  component.
+  The output mask contains a QC parameters aQCPrefix" LOWLIMIT" and
+  aQCPrefix" THRESHOLD" in its header component.
+
+  The function muse_autocalib_create_mask() is similar, but tries to select
+  larger contiguous regions, instead of the lowest ones.
 
   @error{set CPL_ERROR_NULL_INPUT\, return NULL, aImage is NULL}
  */
 /*----------------------------------------------------------------------------*/
 muse_mask *
-muse_sky_create_skymask(muse_image *aImage, double aMinFraction, double aFraction,
-                        const char *aQCPrefix)
+muse_sky_create_skymask(muse_image *aImage, double aMinFraction,
+                        double aFraction, const char *aQCPrefix)
 {
   cpl_ensure(aImage, CPL_ERROR_NULL_INPUT, NULL);
 
-  /* Re-normalize fraction: as parameter, it means the fraction of the remaining image,
-     while we need the absolute number here. Example: if
-     minFraction = 0.2, Fraction = 0.8 means: throw away the lowest 20% and then take the
-     lowest 80% of the remaining image. This converts to 84 % of the total image. */
-  aFraction = aMinFraction + aFraction * (1-aMinFraction);
+  /* Re-normalize fraction: as parameter, it means the fraction of the   *
+   * remaining image, while we need the absolute number here.            *
+   * Example: if minFraction = 0.2, Fraction = 0.8 means: throw away     *
+   * the lowest 20% and then take the lowest 80% of the remaining image. *
+   * This converts to 84 % of the total image.                           */
+  double fraction = aMinFraction + aFraction * (1-aMinFraction);
 
-  /* do statistics */
+  /* do statistics, ignoring pre-existing bad pixels */
   muse_image_reject_from_dq(aImage);
-  double t0 = muse_cplimage_get_percentile(aImage->data, aMinFraction);
-  double t1 = muse_cplimage_get_percentile(aImage->data, aFraction);
+  double t0 = muse_cplimage_get_percentile(aImage->data, aMinFraction),
+         t1 = muse_cplimage_get_percentile(aImage->data, fraction);
   cpl_msg_info(__func__, "Creating sky mask for pixels between minimum (%g) and"
                " threshold (%g)", t0, t1);
 
   /* create output mask with pixels between min and threshold */
-  muse_mask *selectedRegions = muse_mask_new();
-  selectedRegions->mask = cpl_mask_threshold_image_create(aImage->data, t0, t1);
-  cpl_mask_not(selectedRegions->mask);
-  cpl_mask_or(selectedRegions->mask, cpl_image_get_bpm(aImage->data));
-  cpl_mask_not(selectedRegions->mask);
+  muse_mask *selected = muse_mask_new();
+  selected->mask = cpl_mask_threshold_image_create(aImage->data, t0, t1);
+  cpl_mask_not(selected->mask);
+  cpl_mask_or(selected->mask, cpl_image_get_bpm(aImage->data));
+  cpl_mask_not(selected->mask);
 
   /* add threshold as QC parameter */
-  selectedRegions->header = cpl_propertylist_duplicate(aImage->header);
+  selected->header = cpl_propertylist_duplicate(aImage->header);
+  cpl_propertylist_erase_regexp(selected->header, "BUNIT|ESO DRS MUSE", 0);
   char keyword[KEYWORD_LENGTH];
+  snprintf(keyword, KEYWORD_LENGTH, "%s LOWLIMIT", aQCPrefix);
+  cpl_propertylist_append_double(selected->header, keyword, t0);
   snprintf(keyword, KEYWORD_LENGTH, "%s THRESHOLD", aQCPrefix);
-  cpl_propertylist_append_double(selectedRegions->header, keyword, t1);
+  cpl_propertylist_append_double(selected->header, keyword, t1);
 
-  return selectedRegions;
-}
+  return selected;
+} /* muse_sky_create_skymask() */
 
 /*----------------------------------------------------------------------------*/
 /**
