@@ -213,10 +213,11 @@ muse_qi_mask_compute(muse_processing *aProcessing,
     }
 
     /* load wavecaltable and tracetable */
-    cpl_table *wavecaltable = muse_table_load(aProcessing,
-                                              MUSE_TAG_WAVECAL_TABLE, nifu);
-    cpl_table *tracetable = muse_table_load(aProcessing,
-                                            MUSE_TAG_TRACE_TABLE, nifu);
+    cpl_table *wavecaltable = muse_processing_load_ctable(aProcessing,
+                                                          MUSE_TAG_WAVECAL_TABLE,
+                                                          nifu),
+              *tracetable = muse_processing_load_ctable(aProcessing,
+                                                        MUSE_TAG_TRACE_TABLE, nifu);
 
     /* create the wavelength map, first always unbinned */
     int binx = muse_pfits_get_binx(image->header),
