@@ -119,10 +119,11 @@ muse_scibasic_per_exposure(muse_processing *aProcessing,
   const double llimN = kMuseUsefulNLambdaMin - FLT_EPSILON,
                llimE = kMuseUsefulELambdaMin - FLT_EPSILON,
                llimA = kMuseUsefulAOLambdaMin - FLT_EPSILON,
+               llimF = kMuseUsefulNFMLambdaMin - FLT_EPSILON,
                hlim = kMuseUsefulLambdaMax + FLT_EPSILON;
   muse_ins_mode mode = muse_pfits_get_mode(pixtable->header);
   if (aParams->crop) {
-    if ((mode == MUSE_MODE_WFM_AO_N) || (mode == MUSE_MODE_NFM_AO_N)) {
+    if (mode == MUSE_MODE_WFM_AO_N) {
       cpl_msg_info(__func__, "Nominal AO mode: cropping the pixel table of IFU "
                    "%d to %.1f...%.1f Angstrom", aParams->nifu, llimA, hlim);
       muse_pixtable_restrict_wavelength(pixtable, llimA, hlim);
@@ -130,6 +131,10 @@ muse_scibasic_per_exposure(muse_processing *aProcessing,
       cpl_msg_info(__func__, "Nominal mode: cropping the pixel table of IFU %d"
                    " to %.1f...%.1f Angstrom", aParams->nifu, llimN, hlim);
       muse_pixtable_restrict_wavelength(pixtable, llimN, hlim);
+    } else if (mode == MUSE_MODE_NFM_AO_N) {
+      cpl_msg_info(__func__, "NFM: cropping the pixel table of IFU %d to "
+                   "%.1f...%.1f Angstrom", aParams->nifu, llimF, hlim);
+      muse_pixtable_restrict_wavelength(pixtable, llimF, hlim);
     } else { /* extended mode */
       cpl_msg_info(__func__, "Extended mode: cropping the pixel table of IFU %d"
                    " to %.1f...%.1f Angstrom", aParams->nifu, llimE, hlim);
