@@ -140,7 +140,14 @@ muse_twilight_reconstruct(muse_processing *aProcessing,
 
   muse_pixtable **pts = cpl_calloc(kMuseNumIFUs, sizeof(muse_pixtable));
   unsigned char nifu;
-  #pragma omp parallel for default(none)                 /* as req. by Ralf */ \
+
+  // FIXME: Removed default(none) clause here, to make the code work with
+  //        gcc9 while keeping older versions of gcc working too without
+  //        resorting to conditional compilation. Having default(none) here
+  //        causes gcc9 to abort the build because of __func__ not being
+  //        specified in a data sharing clause. Adding a data sharing clause
+  //        makes older gcc versions choke.
+  #pragma omp parallel for                                                     \
           shared(aBPars, aCPars, aProcessing, geo, pts)
   for (nifu = 1; nifu <= kMuseNumIFUs; nifu++) {
     cpl_table *trace = muse_processing_load_ctable(aProcessing, MUSE_TAG_TRACE_TABLE, nifu),
