@@ -1,7 +1,7 @@
 # ZAP - Zurich Atmosphere Purge
 #
 # Copyright (c) 2014-2016 Kurt Soto
-# Copyright (c) 2015-2017 Simon Conseil
+# Copyright (c) 2015-2019 Simon Conseil
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -22,51 +22,32 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 # DEALINGS IN THE SOFTWARE.
 
-from __future__ import absolute_import, division, print_function
-try:
-  import esorexplugin
-  from esorexplugin import *
-except ImportError:
-    import_ok = 0
-
-try:
-  import astropy.units as u
-  import logging
-  import numpy as np
-  import os
-  import scipy.ndimage as ndi
-  import sys
-
-  from astropy.io import fits
-  from astropy.utils import minversion
-  from astropy.wcs import WCS
-  from functools import wraps
-  from multiprocessing import cpu_count, Manager, Process
-  from scipy.stats import sigmaclip
-  from sklearn.decomposition import PCA
-  from time import time
-except ImportError:
-    import_ok = 0
-
-
-
-
-#from .version import __version__
-
-import sys 
-import os  
-import json
+import astropy.units as u
+import logging
+import numpy as np
+import os
+import scipy.ndimage as ndi
+import sys
 import warnings
-import argparse
-__version__ = '2.1.dev workflow'
+from astropy.io import fits
+from astropy.wcs import WCS
+from functools import wraps
+from multiprocessing import cpu_count, Manager, Process
+from scipy.stats import sigmaclip
+from sklearn.decomposition import PCA
+from time import time
+import esorexplugin
+from esorexplugin import *
+from pkg_resources import get_distribution, DistributionNotFound
+__version__ = '2.1.dev workflow' #get_distribution('zap').version
 
-__all__ = ['processzap', 'SVDoutput', 'nancleanfits', 'contsubfits', 'Zap',
-           'wmedian', 'SKYSEG']
+__all__ = ['process', 'SVDoutput', 'nancleanfits', 'contsubfits', 'Zap',
+           'SKYSEG', '__version__']
 
 # Limits of the segments in Angstroms. Zap now uses by default only one
 # segment, see below for the old values.
-SKYSEG = [0,10000]
-                       
+SKYSEG = [0, 10000]
+
 # These are the old limits from the original zap. Keeping them for reference,
 # and may be useful for further testing.
 # [0, 5400, 5850, 6440, 6750, 7200, 7700, 8265, 8602, 8731, 9275, 10000]
@@ -75,36 +56,24 @@ SKYSEG = [0,10000]
 NOTCH_FILTER_RANGES = {
     'WFM-AO-E': [5755, 6008],
     'WFM-AO-N': [5805, 5966],
-    'NFM-AO-N': [5780, 6050]
+    'NFM-AO-N': [5780, 6050],
 }
 
 # List of allowed values for cftype (continuum filter)
-CFTYPE_OPTIONS = ('weight', 'median', 'fit', 'none')
+CFTYPE_OPTIONS = ('median', 'fit', 'none')
 
 # Number of available CPUs
-try:
- NCPU =  cpu_count()
+NCPU = cpu_count()
 
- PY2 = sys.version_info[0] == 2
- ASTROPY_LT_1_3 = not minversion('astropy', '1.3')
+logging.basicConfig(format='[%(levelname)s] %(message)s', level=logging.INFO,
+                    stream=sys.stdout)
+logger = logging.getLogger(__name__)
 
- if not PY2:
-    text_type = str
-    string_types = (str,)
- else:
-    text_type = unicode
-    string_types = (str, unicode)
-
- logging.basicConfig(format='[%(levelname)s] %(message)s', level=logging.INFO,
-                     stream=sys.stdout)
- logger = logging.getLogger(__name__)
-except:
- failure=1
 
 # ================= Top Level Functions =================
 try:
- def processzap(musecubefits, outcubefits='DATACUBE_ZAP.fits', clean=True,
-            zlevel='median', cftype='weight', cfwidthSVD=300, cfwidthSP=300,
+ def processzap(cubefits, outcubefits='DATACUBE_ZAP.fits', clean=True,
+            zlevel='median', cftype='median', cfwidthSVD=300, cfwidthSP=300,
             nevals=[], extSVD=None, skycubefits=None, mask=None,
             interactive=False, ncpu=None, pca_class=None, n_components=None,
             overwrite=False, varcurvefits=None):
@@ -113,7 +82,7 @@ try:
     optionally writes the product to an output FITS file.
     Parameters
     ----------
-    musecubefits : str
+    cubefits : str
         Input FITS file, containing a cube with data in the first extension.
     outcubefits : str
         Output FITS file, based on the input one to propagate all header
@@ -127,9 +96,8 @@ try:
     zlevel : str
         Method for the zeroth order sky removal: `none`, `sigclip` or `median`
         (default).
-    cftype : {'weight', 'median', 'fit', 'none'}
-        Method for the continuum filter. For the `weight` method, a zeroth
-        order sky is required (see `zlevel`).
+    cftype : {'median', 'fit', 'none'}
+        Method for the continuum filter.
     cfwidthSVD : int or float
         Window size for the continuum filter, for the SVD computation.
         Default to 300.
@@ -167,14 +135,12 @@ try:
     varcurvefits : str
         Path for the optional output of the explained variance curves.
     """
-    
-    logger.info('Running ZAP')
     logger.info('Running ZAP %s !', __version__)
     t0 = time()
-    if not isinstance(musecubefits, string_types):
+#    if not isinstance(cubefits, str):
+    if not isinstance(cubefits, basestring):
         raise TypeError('The process method only accepts a single datacube '
                         'filename.')
-
     # check if outcubefits/skycubefits exists before beginning
     if not overwrite:
         def _check_file_exists(filename):
@@ -197,10 +163,10 @@ try:
         # cfwidth values differ and extSVD is not given. Otherwise, the SVD
         # will be computed in the _run method, which allows to avoid running
         # twice the zlevel and continuumfilter steps.
-        extSVD = SVDoutput(musecubefits, clean=clean, zlevel=zlevel,
+        extSVD = SVDoutput(cubefits, clean=clean, zlevel=zlevel,
                            cftype=cftype, cfwidth=cfwidthSVD, mask=mask)
 
-    zobj = Zap(musecubefits, pca_class=pca_class, n_components=n_components)
+    zobj = Zap(cubefits, pca_class=pca_class, n_components=n_components)
     zobj._run(clean=clean, zlevel=zlevel, cfwidth=cfwidthSP, cftype=cftype,
               nevals=nevals, extSVD=extSVD)
 
@@ -213,12 +179,12 @@ try:
 
     if varcurvefits is not None:
         zobj.writevarcurve(varcurvefits=varcurvefits, overwrite=overwrite)
-    
+
     zobj.mergefits(outcubefits, overwrite=overwrite)
     logger.info('Zapped! (took %.2f sec.)', time() - t0)
 
 
- def SVDoutput(musecubefits, clean=True, zlevel='median', cftype='weight',
+ def SVDoutput(cubefits, clean=True, zlevel='median', cftype='median',
               cfwidth=300, mask=None, ncpu=None, pca_class=None,
               n_components=None):
     """Performs the SVD decomposition of a datacube.
@@ -227,7 +193,7 @@ try:
     a ``Zap`` which can be given to the :func:`~zap.process` function.
     Parameters
     ----------
-    musecubefits : str
+    cubefits : str
         Input FITS file, containing a cube with data in the first extension.
     clean : bool
         If True (default value), the NaN values are cleaned. Spaxels with more
@@ -236,21 +202,20 @@ try:
     zlevel : str
         Method for the zeroth order sky removal: `none`, `sigclip` or `median`
         (default).
-    cftype : {'weight', 'median', 'fit', 'none'}
-        Method for the continuum filter. For the `weight` method, a zeroth
-        order sky is required (see `zlevel`).
+    cftype : {'median', 'fit', 'none'}
+        Method for the continuum filter.
     cfwidth : int or float
         Window size for the continuum filter, default to 300.
     mask : str
         Path of a FITS file containing a mask (1 for objects, 0 for sky).
     """
-    logger.info('Processing %s to compute the SVD', musecubefits)
+    logger.info('Processing %s to compute the SVD', cubefits)
 
     if ncpu is not None:
         global NCPU
         NCPU = ncpu
 
-    zobj = Zap(musecubefits, pca_class=pca_class, n_components=n_components)
+    zobj = Zap(cubefits, pca_class=pca_class, n_components=n_components)
     zobj._prepare(clean=clean, zlevel=zlevel, cftype=cftype,
                   cfwidth=cfwidth, mask=mask)
     zobj._msvd()
@@ -272,16 +237,16 @@ try:
 
     outhead = _newheader(zobj)
     outhdu = fits.PrimaryHDU(data=cube, header=outhead)
-    write_hdulist_to(outhdu, outfits, overwrite=overwrite)
+    outhdu.writeto(outfits, overwrite=overwrite)
     logger.info('Continuum cube file saved to %s', outfits)
 
 
- def nancleanfits(musecubefits, outfn='NANCLEAN_CUBE.fits', rejectratio=0.25,
+ def nancleanfits(cubefits, outfn='NANCLEAN_CUBE.fits', rejectratio=0.25,
                  boxsz=1, overwrite=False):
     """Interpolates NaN values from the nearest neighbors.
     Parameters
     ----------
-    musecubefits : str
+    cubefits : str
         Input FITS file, containing a cube with data in the first extension.
     outfn : str
         Output FITS file. Default to ``NANCLEAN_CUBE.fits``.
@@ -293,10 +258,10 @@ try:
         Default to 1, which looks for the 26 nearest neighbors which
         is a 3x3x3 cube.
     """
-    with fits.open(musecubefits) as hdu:
+    with fits.open(cubefits) as hdu:
         hdu[1].data = _nanclean(hdu[1].data, rejectratio=rejectratio,
                                 boxsz=boxsz)[0]
-        write_hdulist_to(hdu, outfn, overwrite=overwrite)
+        hdu.writeto(outfn, overwrite=overwrite)
 
 
  def timeit(func):
@@ -358,12 +323,21 @@ try:
         A 1d array containing the result of the zero level subtraction
     """
 
-    def __init__(self, musecubefits, pca_class=None, n_components=None):
-        self.musecubefits = musecubefits
-        with fits.open(musecubefits) as hdul:
-            self.ins_mode = hdul[0].header.get('HIERARCH ESO INS MODE')
-            self.cube = hdul[1].data
-            self.header = hdul[1].header
+    def __init__(self, cubefits, pca_class=None, n_components=None):
+        self.cubefits = cubefits
+        self.ins_mode = None
+
+        with fits.open(cubefits) as hdul:
+            self.instrument = hdul[0].header.get('INSTRUME')
+            if self.instrument == 'MUSE':
+                self.ins_mode = hdul[0].header.get('HIERARCH ESO INS MODE')
+                self.cube = hdul[1].data
+                self.header = hdul[1].header
+            elif self.instrument == 'KCWI':
+                self.cube = hdul[0].data
+                self.header = hdul[0].header
+            else:
+                raise ValueError('unsupported instrument %s' % self.instrument)
 
         # Workaround for floating points errors in wcs computation: if cunit is
         # specified, wcslib will convert in meters instead of angstroms, so we
@@ -387,6 +361,8 @@ try:
                 NOTCH_FILTER_RANGES[self.ins_mode], 0)[0].astype(int)
             lmin, lmax = self.notch_limits
             self.cube[lmin:lmax + 1] = 0.0
+        else:
+            self.notch_limits = None
 
         # NaN Cleaning
         self.run_clean = False
@@ -414,7 +390,7 @@ try:
         # identify the spectral range of the dataset
         laxmin = min(self.laxis)
         laxmax = max(self.laxis)
-        
+
         # List of segmentation limits in the optical
         skyseg = np.array(SKYSEG)
         skyseg = skyseg[(skyseg > laxmin) & (skyseg < laxmax)]
@@ -422,7 +398,7 @@ try:
         # segment limit in angstroms
         self.lranges = (np.vstack([np.append(laxmin - 10, skyseg),
                                    np.append(skyseg, laxmax + 10)])).T
-        logger.info(self.lranges)
+
         # segment limit in pixels
         laxis = self.laxis
         lranges = self.lranges
@@ -431,7 +407,7 @@ try:
             paxis = wlaxis[(laxis > lranges[i, 0]) & (laxis <= lranges[i, 1])]
             pranges.append((np.min(paxis), np.max(paxis) + 1))
         self.pranges = np.array(pranges)
-	logger.info(self.pranges)
+
         # eigenspace Subset
         if pca_class is not None:
             logger.info('Using %s', pca_class)
@@ -445,12 +421,8 @@ try:
         self.cleancube = None
 
     @timeit
-    def _prepare(self, clean=True, zlevel='median', cftype='weight',
+    def _prepare(self, clean=True, zlevel='median', cftype='median',
                  cfwidth=300, extzlevel=None, mask=None):
-        # Check for consistency between weighted median and zlevel keywords
-        if cftype == 'weight' and zlevel == 'none':
-            raise ValueError('Weighted median requires a zlevel calculation')
-
         # clean up the nan values
         if clean:
             self._nanclean()
@@ -475,7 +447,7 @@ try:
         # normalize the variance in the segments.
         self._normalize_variance()
 
-    def _run(self, clean=True, zlevel='median', cftype='weight',
+    def _run(self, clean=True, zlevel='median', cftype='median',
              cfwidth=300, nevals=[], extSVD=None):
         """ Perform all steps to ZAP a datacube:
         - NaN re/masking,
@@ -512,7 +484,7 @@ try:
 
         # stuff the new spectra back into the cube
         self.remold()
-        
+
     def _nanclean(self):
         """
         Detects NaN values in cube and removes them by replacing them with an
@@ -583,7 +555,7 @@ try:
         self.stack -= self.zlsky[:, np.newaxis]
 
     @timeit
-    def _continuumfilter(self, cfwidth=300, cftype='weight'):
+    def _continuumfilter(self, cfwidth=300, cftype='median'):
         """A multiprocessed implementation of the continuum removal.
         This process distributes the data to many processes that then
         reassemble the data.  Uses two filters, a small scale (less than the
@@ -595,22 +567,24 @@ try:
             removed
         """
         if cftype not in CFTYPE_OPTIONS:
-            raise ValueError("cftype must be weight, median, fit or none, "
-                             "got {}".format(cftype))
+            raise ValueError("cftype must be median, fit or none, got {}"
+                             .format(cftype))
         logger.info('Applying Continuum Filter, cftype=%s', cftype)
         self._cftype = cftype
         self._cfwidth = cfwidth
 
-        weight = None
-        if cftype == 'weight':
-            weight = np.abs(self.zlsky - (np.max(self.zlsky) + 1))
-
         # remove continuum features
         if cftype == 'none':
             self.normstack = self.stack.copy()
         else:
+            if cftype == 'fit' and self.instrument != 'MUSE':
+                warnings.warn('the continuum fit method is currently adapted '
+                              'to MUSE and should not be used for other '
+                              'instruments', UserWarning)
+
             self.contarray = _continuumfilter(self.stack, cftype,
-                                              weight=weight, cfwidth=cfwidth)
+                                              cfwidth=cfwidth,
+                                              notch_limits=self.notch_limits)
             self.normstack = self.stack - self.contarray
 
     def _normalize_variance(self):
@@ -719,7 +693,7 @@ try:
         logger.info('Applying correction and reshaping data product')
         self.cleancube = self.make_cube_from_stack(self.stack - self.recon,
                                                    with_nans=self.run_clean)
-        logger.info('done')
+
     def reprocess(self, nevals=[]):
         """ A method that redoes the eigenvalue selection, reconstruction, and
         remolding of the data.
@@ -774,7 +748,7 @@ try:
         """Write the processed datacube to an individual fits file."""
         outhead = _newheader(self)
         outhdu = fits.PrimaryHDU(data=self.cleancube, header=outhead)
-        write_hdulist_to(outhdu, outcubefits, overwrite=overwrite)
+        outhdu.writeto(outcubefits, overwrite=overwrite)
         logger.info('Cube file saved to %s', outcubefits)
 
     def writeskycube(self, skycubefits='SKYCUBE_ZAP.fits', overwrite=False):
@@ -782,7 +756,7 @@ try:
         outcube = self.cube - self.cleancube
         outhead = _newheader(self)
         outhdu = fits.PrimaryHDU(data=outcube, header=outhead)
-        write_hdulist_to(outhdu, skycubefits, overwrite=overwrite)
+        outhdu.writeto(skycubefits, overwrite=overwrite)
         logger.info('Sky cube file saved to %s', skycubefits)
 
     def writevarcurve(self, varcurvefits='VARCURVE_ZAP.fits', overwrite=False):
@@ -791,17 +765,24 @@ try:
         table = Table([m.explained_variance_ for m in self.models])
         hdu = fits.table_to_hdu(table)
         _newheader(self, hdu.header)
-        write_hdulist_to(hdu, varcurvefits, overwrite=overwrite)
+        hdu.writeto(varcurvefits, overwrite=overwrite)
         logger.info('Variance curve file saved to %s', varcurvefits)
 
     def mergefits(self, outcubefits, overwrite=False):
         """Merge the ZAP cube into the full muse datacube and write."""
         # make sure it has the right extension
         outcubefits = outcubefits.split('.fits')[0] + '.fits'
-        with fits.open(self.musecubefits) as hdu:
-            hdu[1].header = _newheader(self)
-            hdu[1].data = self.cleancube
-            write_hdulist_to(hdu, outcubefits, overwrite=overwrite)
+        with fits.open(self.cubefits) as hdu:
+            if self.instrument == 'MUSE':
+                hdu[1].header = _newheader(self)
+                hdu[1].data = self.cleancube
+            elif self.instrument == 'KCWI':
+                hdu[0].header = _newheader(self)
+                hdu[0].data = self.cleancube
+            else:
+                raise ValueError('unsupported instrument %s' % self.instrument)
+
+            hdu.writeto(outcubefits, overwrite=overwrite)
         logger.info('Cube file saved to %s', outcubefits)
 
     def plotvarcurve(self, i=0, ax=None):
@@ -842,16 +823,7 @@ try:
             self.plotvarcurve(i=i, ax=axes[i])
 
 
-# ================= Helper Functions =================
-
- if ASTROPY_LT_1_3:
-    # the 'clobber' parameter was renamed to 'overwrite' in 1.3
-    def write_hdulist_to(hdulist, fileobj, overwrite=False, **kwargs):
-        hdulist.writeto(fileobj, clobber=overwrite, **kwargs)
- else:
-    def write_hdulist_to(hdulist, fileobj, overwrite=False, **kwargs):
-        hdulist.writeto(fileobj, overwrite=overwrite, **kwargs)
-
+ # ================= Helper Functions =================
 
  def worker(f, i, chunk, out_q, err_q, kwargs):
     try:
@@ -865,8 +837,7 @@ try:
 
 
  def parallel_map(func, arr, indices, **kwargs):
-    logger.debug('Running function %s with indices: %s',
-                 func.__name__, indices)
+    logger.debug('Running function %s with %s chunks', func.__name__, indices)
     axis = kwargs.pop('axis', None)
     if isinstance(indices, (int, np.integer)) and indices == 1:
         return [func(0, arr, **kwargs)]
@@ -893,11 +864,8 @@ try:
     for proc in jobs:
         proc.join()
 
-
     if not err_q.empty():
         # kill all on any exception from any one slave
-        logger.info(err_q.get())
-
         raise err_q.get()
 
     # Processes finish in arbitrary order. Process IDs double
@@ -906,7 +874,7 @@ try:
     while not out_q.empty():
         idx, result = out_q.get()
         results[idx] = result
- 
+
     return results
 
 
@@ -920,32 +888,50 @@ try:
     return deriv, mn1, std1
 
 
- def _continuumfilter(stack, cftype, weight=None, cfwidth=300):
+ def _continuumfilter(stack, cftype, cfwidth=300, notch_limits=None):
     if cftype == 'fit':
         x = np.arange(stack.shape[0])
+
         # Excluding the very red part for the fit. This is Muse-specific,
         # but anyway for another instrument this method should probably
         # not be used as is.
-        res = np.polynomial.polynomial.polyfit(x[:3600], stack[:3600], deg=5)
+        w = np.ones(stack.shape[0])
+        w[3600:] = 0
+
+        if notch_limits is not None:
+            # Exclude the notch filter region
+            lmin, lmax = notch_limits
+            w[lmin:lmax + 1] = 0
+
+        res = np.polynomial.polynomial.polyfit(x, stack, deg=5, w=w)
         ret = np.polynomial.polynomial.polyval(x, res, tensor=True)
         return ret.T
 
     if cftype == 'median':
         func = _icfmedian
-        weight = None
-    elif cftype == 'weight':
-        func = _icfweight
+    else:
+        raise ValueError('unknown cftype option')
+
     logger.info('Using cfwidth=%d', cfwidth)
-    c = parallel_map(func, stack, NCPU, axis=1, weight=weight, cfwidth=cfwidth)
-    return np.concatenate(c, axis=1)
+
+    if notch_limits is not None:
+        # To manage the notch filter which is filled with zeros, we process the
+        # stack in two halves, before and after the filter.
+        c = np.zeros_like(stack)
+        ctmp = parallel_map(func, stack[:notch_limits[0]], NCPU, axis=1,
+                            cfwidth=cfwidth)
+        c[:notch_limits[0]] = np.concatenate(ctmp, axis=1)
+        ctmp = parallel_map(func, stack[notch_limits[1]:], NCPU, axis=1,
+                            cfwidth=cfwidth)
+        c[notch_limits[1]:] = np.concatenate(ctmp, axis=1)
+    else:
+        c = parallel_map(func, stack, NCPU, axis=1, cfwidth=cfwidth)
+        c = np.concatenate(c, axis=1)
+
+    return c
 
 
- def _icfweight(i, stack, weight=None, cfwidth=None):
-    return np.array([wmedian(row, weight, cfwidth=cfwidth)
-                     for row in stack.T]).T
-
-
- def _icfmedian(i, stack, weight=None, cfwidth=None):
+ def _icfmedian(i, stack, cfwidth=None):
     ufilt = 3  # set this to help with extreme over/under corrections
     return ndi.median_filter(
         ndi.uniform_filter(stack, (ufilt, 1)), (cfwidth, 1))
@@ -957,58 +943,6 @@ try:
     return np.lib.stride_tricks.as_strided(a, shape=shape, strides=strides)
 
 
- def wmedian(spec, wt, cfwidth=300):
-    """Performs a weighted median filtering of a 1d spectrum.
-    Operates using a cumulative sum curve.
-    Parameters
-    ----------
-    spec : numpy.ndarray
-        Input 1d spectrum to be filtered
-    wt : numpy.ndarray
-        A spectrum of equal length as the input array to provide the weights.
-    cfwidth : int or float
-        Window size for the continuum filter, for the SVD computation.
-        Default to 300.
-    """
-    # ignore the warning (feature not a bug)
-    old_settings = np.seterr(divide='ignore')
-    spec = np.pad(spec, (cfwidth, cfwidth), 'constant', constant_values=0)
-    wt = np.abs(wt)
-    wt = np.pad(wt, (cfwidth, cfwidth), 'constant',
-                constant_values=(np.min(wt) / 1000., np.min(wt) / 1000.))
-
-    # do some striding for speed
-    swin = rolling_window(spec, cfwidth)  # create window container array
-    wwin = rolling_window(wt, cfwidth)  # create window container array
-
-    # sort based on data
-    srt = np.argsort(swin, axis=-1)
-    ind = np.ogrid[0:swin.shape[0], 0:swin.shape[1]]
-    sdata = swin[ind[0], srt]
-    swt = wwin[ind[0], srt]
-
-    # calculate accumulated weights
-    awt = np.cumsum(swt, axis=-1)
-
-    # new weightsort for normalization and consideration of data
-    nw = (awt - 0.5 * swt) / awt[:, -1][:, np.newaxis]
-
-    # find the midpoint in the new weight sort
-    s = np.argmin(np.abs(nw - 0.5), axis=-1)
-    sl = np.arange(len(s))
-    nws = nw[sl, s]
-    nws1 = nw[sl, s - 1]
-
-    f1 = (nws - 0.5) / (nws - nws1)
-    f2 = (0.5 - nws1) / (nws - nws1)
-    wmed = sdata[sl, s - 1] * f1 + sdata[sl, s] * f2
-    width = cfwidth // 2
-    wmed = wmed[width:-width - 1]
-    np.seterr(old_settings['divide'])
-
-    return wmed
-
-
  def _newheader(zobj, header=None):
     """Put the pertinent zap parameters into the header"""
     header = header or zobj.header.copy()
@@ -1100,7 +1034,7 @@ except:
 
 class MuseZap(esorexplugin.RecipePlugin):
     # The recipe must be given a name by setting the 'name' class attribute.
-    name = "muse_zap"     
+    name = "muse_zap"
 
     # A version needs to be assigned. For this it is best to use the
     # VersionNumber method that will correctly encode a major, minor and micro
@@ -1141,19 +1075,19 @@ class MuseZap(esorexplugin.RecipePlugin):
     #       "test"]
     #           "extSVD"]
 
-    fullnames= map(lambda x: "muse_zap.muse_zap."+x, names)
+    fullnames = list(map(lambda x: "muse_zap.muse_zap."+x, names))
 
     desc1="Method for the zeroth order sky removal: 'none', 'sigclip' or 'median' (default)."
     desc2="Method for the continuum filter: 'median' or 'weight' (default). For the 'weight' method, a zeroth order sky is required (see 'zlevel')."
     desc3="Window size for the continuum filter, for the SVD computation. Default to 500."
     desc4= "Window size for the continuum filter used to remove the continuum features for calculating the eigenvalues per spectrum. Smaller values better trace the sources. An optimal range of is typically 20 - 50 pixels. Default to 30."
 
- 
+
         #Provide either a single value
         #that will be used for all of the segments, or a list of 11 values that
         #will be used for each of the segments."
 
-    desc7= "Allow to specify the number of eigenspectra used for each segment. Ignored if pevals is used or optimizetType different from None." 
+    desc7= "Allow to specify the number of eigenspectra used for each segment. Ignored if pevals is used or optimizetType different from None."
         #Provide either a single value
         #that will be used for all of the segments, or a list of 11 values that
         #will be used for each of the segments.
@@ -1166,12 +1100,12 @@ class MuseZap(esorexplugin.RecipePlugin):
  #   desc_test="test parameter"
     parameters = [
         EnumParameter(fullnames[0],'median', ['median', 'none', 'sigclip'],cli_alias=alias[0],cli_enabled=True,description=desc1),
-        EnumParameter(fullnames[1],'weight', ['weight', 'median'],cli_alias=alias[1],cli_enabled=True,description=desc2),
+        EnumParameter(fullnames[1],'median', ['fit', 'median','none'],cli_alias=alias[1],cli_enabled=True,description=desc2),
         ValueParameter(fullnames[2],500,cli_alias=alias[2],cli_enabled=True, description=desc3),
         ValueParameter(fullnames[3],30,cli_alias=alias[3],cli_enabled=True, description=desc4),
         ValueParameter(fullnames[4],-1  ,cli_alias=alias[4],cli_enabled=True, description=desc7),
         ValueParameter(fullnames[5],True,cli_alias=alias[5],cli_enabled=True, description=desc8)]
-      #  EnumParameter(fullnames[6],1.0,[0.,1.0,10.],cli_alias=alias[6],cli_enabled=True, description=desc_test)]
+      ##  EnumParameter(fullnames[6],1.0,[0.,1.0,10.],cli_alias=alias[6],cli_enabled=True, description=desc_test)]
 
     def set_frame_group(self, frame):
         if frame.tag == 'RAW':
@@ -1189,62 +1123,62 @@ class MuseZap(esorexplugin.RecipePlugin):
 
         mask_cube_filename=None
         mask_sky_filename=None
-        IsThereSkyCube = 0 
+        IsThereSkyCube = 0
         IsThereObjectCube = 0
-  
+
         for j in range(0,len(frames)):
             if frames[j].tag == 'MASK_FINAL_CUBE':
                 mask_filename = frames[j].filename
             if frames[j].tag == 'DATACUBE_SKY':
-                IsThereSkyCube = 1 
-                muse_sky_cubefits_filename = frames[j].filename                
+                IsThereSkyCube = 1
+                muse_sky_cubefits_filename = frames[j].filename
             if frames[j].tag == 'DATACUBE_OBJECT':
                 IsThereObjectCube = 1
                 muse_object_cubefits_filename = frames[j].filename
-               
-        
-        outcubefits_filename = 'DATACUBE_CLEANED.fits'          
+
+
+        outcubefits_filename = 'DATACUBE_CLEANED.fits'
         if IsThereSkyCube ==1:
           IsThereObjectCube = 0
           extSVD = SVDoutput(muse_sky_cubefits_filename, mask=mask_filename,
                   clean=clean, zlevel=zlevel, cftype=cftype,
                   cfwidth=cfwidthSVD,ncpu=NCPU)
-  
-          
+
+
           try:
             if nevals == -1:
                nevals = []
-            
+
             processzap(muse_object_cubefits_filename, outcubefits=outcubefits_filename,
                        clean=clean,zlevel=zlevel,cftype=cftype, cfwidthSVD=cfwidthSVD,
-                       cfwidthSP=cfwidthSP,  nevals=nevals,ncpu=NCPU, 
+                       cfwidthSP=cfwidthSP,  nevals=nevals,ncpu=NCPU,
                        extSVD=extSVD, skycubefits=None, interactive=False)
-                 
-          except: 
+
+          except:
             self.raise_error(' execution error, check if products already exist')
 
-          
+
         if IsThereObjectCube ==1:
-          
-          try:          
+
+          try:
              if nevals == -1:
                 nevals = []
              processzap(muse_object_cubefits_filename, outcubefits=outcubefits_filename,clean=clean,zlevel=zlevel,
-                              cftype=cftype, cfwidthSVD=cfwidthSVD, cfwidthSP=cfwidthSP, nevals=nevals,ncpu=NCPU, 
+                              cftype=cftype, cfwidthSVD=cfwidthSVD, cfwidthSP=cfwidthSP, nevals=nevals,ncpu=NCPU,
                               mask=mask_filename, interactive=False)
-                                            
+
           except:
                 self.raise_error(' execution error, check if products already exist')
 
 
-        
+
         hdu = fits.open(outcubefits_filename,method='update')
         hdu[0].header['HIERARCH ESO PRO CATG'] = 'DATACUBE_CLEANED'
         if IsThereSkyCube ==1:
-	    s1=muse_sky_cubefits_filename.split("/")[-1]
-	    s2=mask_filename.split("/")[-1]
-	    s1=s1[:50]
-	    s2=s2[:50]
+            s1=muse_sky_cubefits_filename.split("/")[-1]
+            s2=mask_filename.split("/")[-1]
+            s1=s1[:50]
+            s2=s2[:50]
             hdu[0].header['HIERARCH ESO SKYCUBE'] = s1
             hdu[0].header['HIERARCH ESO SKYCUBE MASK'] = s2
         else:
@@ -1252,22 +1186,22 @@ class MuseZap(esorexplugin.RecipePlugin):
             hdu[0].header['HIERARCH ESO SKYCUBE MASK'] = 'None'
         if IsThereObjectCube ==1:
             s3 = mask_filename.split("/")[-1]
-	    s3=s3[:50]
+            s3=s3[:50]
             hdu[0].header['HIERARCH ESO OBJECTCUBE MASK'] = s3
         else:
             hdu[0].header['HIERARCH ESO OBJECTCUBE MASK'] = 'None'
-          
+
         for k in range(len(self.input_parameters)):
             hkey1 = 'HIERARCH ESO ZAP REC'+str(k+1)+' NAME'
             hkey2 = 'HIERARCH ESO ZAPn REC'+str(k+1)+' VALUE'
             hdu[0].header[hkey1] = self.alias[k]
             hdu[0].header[hkey2] = values[k]
 
-        outcubefits = Frame(outcubefits_filename,'DATACUBE_CLEANED', type = Frame.TYPE_IMAGE) 
+        outcubefits = Frame(outcubefits_filename,'DATACUBE_CLEANED', type = Frame.TYPE_IMAGE)
         output_frames.append(outcubefits)
-        
+
         outcubefits.write(hdu,overwrite = True, output_verify = 'ignore',checksum=True)
-                
+
 
 
         return output_frames
