@@ -395,7 +395,7 @@ class DataPlotterManager(object):
             frame["datasets"] = datasets
             self.frameset[fitsfile.category].append(frame)
 
-        if self.tagInputData in self.frameset:
+        if self.tagInputData in self.frameset and self.tagAuxiliaryData in self.frameset:
             self._plt_creator = self._data_plot_create
             self._plt_plotter = self._data_plot_draw
         else:
@@ -681,12 +681,12 @@ class DataPlotterManager(object):
 
 
     def _dummy_plot_draw(self):
-        label = "Data not found! Input files should contain these types:\n%s" \
-            % self.tagInputData
+        label = "Data not found!\n\nInput files should contain these types:\n%s\n%s" \
+            % (self.tagInputData, self.tagAuxiliaryData)
 
         self._plt["dummy"].set_axis_off()
-        self._plt["dummy"].text(0.1, 0.6, label, color="#11557c", fontsize=18,
-                                horizontalalignment="left", verticalalignment="center",
+        self._plt["dummy"].text(0.5, 0.5, label, color="#11557c", fontsize=18,
+                                horizontalalignment="center", verticalalignment="center",
                                 alpha=0.25)
         self._plt["dummy"].tooltip = "No data found"
         return
