@@ -34,6 +34,8 @@
   This program explicitely tests
     muse_cplimage_slope_window
     muse_cplmask_adapt_to_image
+    muse_cplmask_fill_window
+    muse_cplimage_copy_within_mask
     muse_cplvector_get_adev_const
     muse_cplvector_get_median_dev
     muse_cplarray_erase_outliers
@@ -188,9 +190,106 @@ int main(int argc, char **argv)
   cpl_test_null(muse_cplmask_adapt_to_image(mask, image));
   cpl_test(cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
   cpl_errorstate_set(state);
-  cpl_mask_delete(mask);
   cpl_image_delete(image);
 
+  /* test muse_cplmask_fill_window() */
+  state = cpl_errorstate_get();
+  int nmasked = cpl_mask_count(mask);
+  /* set 20 pixels to 1 (by chance, they don't overlap with existing) */
+  cpl_error_code rc = muse_cplmask_fill_window(mask, 1, 1, 5, 4, 1);
+  cpl_test(cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NONE);
+  cpl_test_eq(nmasked + 20, cpl_mask_count(mask));
+ /* set 20 pixels to 0 again */
+  rc = muse_cplmask_fill_window(mask, 1, 1, 5, 4, 0);
+  cpl_test(cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NONE);
+  cpl_test_eq(nmasked, cpl_mask_count(mask));
+  /* failure cases */
+  rc = muse_cplmask_fill_window(NULL, 1, 1, 5, 4, 1);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  rc = muse_cplmask_fill_window(mask, 0, 1, 5, 4, 1);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_ILLEGAL_INPUT);
+  cpl_errorstate_set(state);
+  rc = muse_cplmask_fill_window(mask, 1, 0, 5, 4, 1);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_ILLEGAL_INPUT);
+  cpl_errorstate_set(state);
+  rc = muse_cplmask_fill_window(mask, 1, 1, 1001, 4, 1);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_ILLEGAL_INPUT);
+  cpl_errorstate_set(state);
+  rc = muse_cplmask_fill_window(mask, 1, 1, 5, 1001, 1);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_ILLEGAL_INPUT);
+  cpl_errorstate_set(state);
+  rc = muse_cplmask_fill_window(mask, 2, 1, 1, 4, 1);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_ILLEGAL_INPUT);
+  cpl_errorstate_set(state);
+  rc = muse_cplmask_fill_window(mask, 1, 5, 5, 4, 1);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_ILLEGAL_INPUT);
+  cpl_errorstate_set(state);
+  /* all the above should have failed, i.e. not changed the mask */
+  cpl_test_eq(nmasked, cpl_mask_count(mask));
+  cpl_mask_delete(mask);
+
+  /* test muse_cplimage_copy_within_mask() */
+  cpl_image *im1 = cpl_image_new(6, 6, CPL_TYPE_FLOAT),
+            *im2 = cpl_image_new(6, 6, CPL_TYPE_FLOAT),
+            *im3 = cpl_image_new(5, 5, CPL_TYPE_FLOAT),
+            *im4 = cpl_image_new(6, 6, CPL_TYPE_INT),
+            *im5 = cpl_image_new(6, 6, CPL_TYPE_DOUBLE);
+  mask = cpl_mask_new(6, 6);
+  cpl_image_fill_window(im2, 1, 1, 6, 6, 1.);
+  /* mask the whole part, except two pixels */
+  muse_cplmask_fill_window(mask, 1, 1, 6, 6, 1);
+  cpl_mask_set(mask, 1, 1, 0);
+  cpl_mask_set(mask, 3, 4, 0);
+  rc = muse_cplimage_copy_within_mask(im1, im2, mask);
+  cpl_test(cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NONE);
+  /* should have copied over two pixels with value 1 */
+  cpl_test_abs(cpl_image_get_mean(im1), 2./36., DBL_EPSILON);
+  int err;
+  cpl_test(cpl_image_get(im1, 1, 1, &err) == 1.);
+  cpl_test(cpl_image_get(im1, 3, 4, &err) == 1.);
+  /* failure cases */
+  rc = muse_cplimage_copy_within_mask(NULL, im2, mask);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  rc = muse_cplimage_copy_within_mask(im1, NULL, mask);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  rc = muse_cplimage_copy_within_mask(im1, im2, NULL);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  /* wrong sizes */
+  rc = muse_cplimage_copy_within_mask(im3, im2, mask);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_INCOMPATIBLE_INPUT);
+  cpl_errorstate_set(state);
+  rc = muse_cplimage_copy_within_mask(im1, im3, mask);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_INCOMPATIBLE_INPUT);
+  cpl_errorstate_set(state);
+  cpl_mask *mask2 = cpl_mask_new(4, 4);
+  rc = muse_cplimage_copy_within_mask(im1, im2, mask2);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_INCOMPATIBLE_INPUT);
+  cpl_errorstate_set(state);
+  cpl_mask_delete(mask2);
+  /* wrong types */
+  rc = muse_cplimage_copy_within_mask(im4, im2, mask);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_INVALID_TYPE);
+  cpl_errorstate_set(state);
+  rc = muse_cplimage_copy_within_mask(im5, im2, mask);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_INVALID_TYPE);
+  cpl_errorstate_set(state);
+  rc = muse_cplimage_copy_within_mask(im1, im4, mask);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_INVALID_TYPE);
+  cpl_errorstate_set(state);
+  rc = muse_cplimage_copy_within_mask(im1, im5, mask);
+  cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_INVALID_TYPE);
+  cpl_errorstate_set(state);
+  cpl_image_delete(im1);
+  cpl_image_delete(im2);
+  cpl_image_delete(im3);
+  cpl_image_delete(im4);
+  cpl_image_delete(im5);
+  cpl_mask_delete(mask);
+
   /* test muse_cplvector_get_adev_const() */
   double vdata1[] = { 1, 2, 3, 4, 5 }, /* median = 3., dev = 1.2 */
          vdata2[] = { -8, -2, 11, 1, 10, 4 }; /* median = 2.5, dev = 34./6. ~ 5.667 */
@@ -429,7 +528,6 @@ int main(int argc, char **argv)
   cpl_test_rel(cpl_array_get_double(adouble, 1, NULL), 2.775, DBL_EPSILON);
   cpl_test_rel(cpl_array_get_double(adouble, 2, NULL), 123., DBL_EPSILON);
   cpl_test_rel(cpl_array_get_double(adouble, 3, NULL), -12., DBL_EPSILON);
-  int err;
   cpl_array_get_double(adouble, 4, &err);
   cpl_test_eq(err, 1); /* invalid element */
   cpl_array_delete(a);
@@ -652,11 +750,38 @@ int main(int argc, char **argv)
   cpl_test(cpl_propertylist_get_int(header, "BLAI") == 10);
   cpl_test(cpl_propertylist_get_long(header, "BLAL") == 20);
   cpl_test(cpl_propertylist_get_long_long(header, "BLALL") == 30);
+  cpl_propertylist_delete(header);
+
+    /* test muse_cplpropertylist_update_fp() */
+  state = cpl_errorstate_get();
+  header = cpl_propertylist_new();
+  cpl_test(muse_cplpropertylist_update_fp(NULL, "BLA BLA", 0.)
+           == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  cpl_test(muse_cplpropertylist_update_fp(header, NULL, 0.)
+           == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
+  cpl_test(muse_cplpropertylist_update_fp(header, "BLA BLA", 13.)
+           == CPL_ERROR_NONE);
+  /* set some integers */
+  cpl_propertylist_append_float(header, "BLAF", 1);
+  cpl_propertylist_append_double(header, "BLAD", 1);
+  state = cpl_errorstate_get();
+  /* reset the integer values */
+  cpl_test(muse_cplpropertylist_update_fp(header, "BLAF", 10)
+           == CPL_ERROR_NONE);
+  cpl_test(muse_cplpropertylist_update_fp(header, "BLAD", 20)
+           == CPL_ERROR_NONE);
+  cpl_test(cpl_errorstate_is_equal(state));
+  /* read the values back */
+  cpl_test(cpl_propertylist_get_float(header, "BLAF") == 10);
+  cpl_test(cpl_propertylist_get_double(header, "BLAD") == 20);
+  cpl_test(cpl_propertylist_get_float(header, "BLA BLA") == 13.);
   cpl_propertylist_delete(header);
 
   /* test muse_cplframeset_erase_all() */
   state = cpl_errorstate_get();
-  cpl_error_code rc = muse_cplframeset_erase_all(NULL);
+  rc = muse_cplframeset_erase_all(NULL);
   cpl_test(!cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NULL_INPUT);
   cpl_errorstate_set(state);
   /* create some frame set */
