@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2007-2014 European Southern Observatory
+ * Copyright (C) 2007-2015 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
     muse_quality_set_saturated
     muse_quality_image_reject_using_dq
     muse_quality_dark_badpix
+    muse_quality_dark_refine_badpix
     muse_quality_bad_columns
     muse_quality_flat_badpix
     muse_quality_convert_dq
@@ -304,6 +305,20 @@ int main(int argc, char **argv)
            == EURO3D_DEADPIXEL); /* Q4, 12.2 sigma */
   cpl_test(cpl_image_get(dark->dq, 1021, 2133, &err)
            == EURO3D_DEADPIXEL); /* Q4, 15.5 sigma */
+  /* also test muse_quality_dark_refine_badpix() with this */
+  state = cpl_errorstate_get();
+  int sum0 = cpl_image_get_mean_window(dark->dq, 976, 1891, 1023, 2133) * 11664;
+  int nrefined1 = muse_quality_dark_refine_badpix(dark, 3., 1);
+  int sum1 = cpl_image_get_mean_window(dark->dq, 976, 1891, 1023, 2133) * 11664;
+  int nrefined2 = muse_quality_dark_refine_badpix(dark, 3., 2);
+  cpl_test(nrefined1 > 0 && cpl_errorstate_is_equal(state));
+  cpl_test_zero(nrefined2); /* no new ones with extra iteration */
+  /* compared to the previous case, the region with the charge distributed *
+   * fromthe bad column should have added ~3700 pixels marked hot (=256)   */
+  cpl_test((sum1 - sum0 > 3600 * 256) && (sum1 - sum0 < 3800 * 256));
+  cpl_msg_debug(__func__, "%d/%d refined (%d added to summed DQ)", nrefined1,
+                nrefined2, sum1 - sum0);
+  /* continue with special/failure cases */
   cpl_image_delete(dark->dq);
   dark->dq = cpl_image_duplicate(dq);
   cpl_image_accept_all(dark->data);
@@ -315,6 +330,12 @@ int main(int argc, char **argv)
   dark->data = NULL;
   state = cpl_errorstate_get();
   ndarkbad = muse_quality_dark_badpix(dark, 0., 5.); /* fail without data */
+  cpl_test(ndarkbad == -2 && !cpl_errorstate_is_equal(state) &&
+           cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
+  cpl_msg_debug(__func__, "%d ndarkbad (%d)", ndarkbad, cpl_error_get_code());
+  cpl_errorstate_set(state);
+  nrefined1 = muse_quality_dark_refine_badpix(dark, 3., 1);
+  cpl_msg_debug(__func__, "%d nrefined (%d)", nrefined1, cpl_error_get_code());
   cpl_test(ndarkbad == -2 && !cpl_errorstate_is_equal(state) &&
            cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
   cpl_errorstate_set(state);
@@ -326,6 +347,10 @@ int main(int argc, char **argv)
   cpl_test(ndarkbad == -2 && !cpl_errorstate_is_equal(state) &&
            cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
   cpl_errorstate_set(state);
+  nrefined1 = muse_quality_dark_refine_badpix(dark, 3., 1);
+  cpl_test(ndarkbad == -2 && !cpl_errorstate_is_equal(state) &&
+           cpl_error_get_code() == CPL_ERROR_ILLEGAL_INPUT);
+  cpl_errorstate_set(state);
   muse_image_delete(dark);
   cpl_image_delete(dq);
   state = cpl_errorstate_get();
@@ -333,6 +358,10 @@ int main(int argc, char **argv)
   cpl_test(ndarkbad == -1 && !cpl_errorstate_is_equal(state) &&
            cpl_error_get_code() == CPL_ERROR_NULL_INPUT);
   cpl_errorstate_set(state);
+  nrefined1 = muse_quality_dark_refine_badpix(NULL, 3., 1);
+  cpl_test(ndarkbad == -1 && !cpl_errorstate_is_equal(state) &&
+           cpl_error_get_code() == CPL_ERROR_NULL_INPUT);
+  cpl_errorstate_set(state);
 
   /* test bad column detection on bias data */
   state = cpl_errorstate_get();
@@ -409,7 +438,7 @@ int main(int argc, char **argv)
 
   /* try bad pixel detection on flat-field data */
   muse_image *flat = muse_image_load(BASEFILENAME"_flat.fits");
-  cpl_table *trace = cpl_table_load(BASEFILENAME"_trace.fits", 1, 0);
+  cpl_table *trace = cpl_table_load(BASEFILENAME"_trace.fits", 1, 1);
   if (!flat || !trace) {
     cpl_msg_error(__func__, "failed to load flat image or trace table: %s",
                   cpl_error_get_message());
